@@ -52,7 +52,7 @@
 #define HAVE_ERRNO_H 1
 
 /* We don't have popen, but we do have _popen */
-#define HAVE_POPEN 1
+/* #define HAVE_POPEN 1 */
 
 /* We do have perror */
 #define HAVE_PERROR 1
@@ -62,3 +62,6 @@
 
 /* We do not have unistd.h. */
 #define YY_NO_UNISTD_H 1
+
+/* Extension for executables */
+#define EXEEXT ".exe"
