@@ -34,7 +34,10 @@
 /* #undef FIXED16 */
 
 /* Use fixed-point computation */
-#define FIXED_POINT 1
+/* #define FIXED_POINT 1 */
 
 /* The size of `long', as computed by sizeof. */
 #define SIZEOF_LONG 4
+
+/* Extension for executables */
+#define EXEEXT ".exe"
