@@ -49,14 +49,15 @@
 #include <config.h>
 #endif
 
-#include "prim_type.h"
-#include "byteorder.h"
-#include "fixpoint.h"
+#include "sphinxbase/prim_type.h"
+#include "sphinxbase/byteorder.h"
+#include "sphinxbase/fixpoint.h"
+#include "sphinxbase/genrand.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/cmd_ln.h"
+#include "sphinxbase/ckd_alloc.h"
+
 #include "fe_internal.h"
-#include "genrand.h"
-#include "err.h"
-#include "cmd_ln.h"
-#include "ckd_alloc.h"
 #include "fe_warp.h"
 
 static const arg_t fe_args[] = {
@@ -67,11 +68,19 @@ static const arg_t fe_args[] = {
 int
 fe_parse_general_params(cmd_ln_t *config, fe_t * fe)
 {
-    int j;
+    int j, frate;
 
     fe->config = config;
     fe->sampling_rate = cmd_ln_float32_r(config, "-samprate");
-    fe->frame_rate = (int16)cmd_ln_int32_r(config, "-frate");
+    frate = cmd_ln_int32_r(config, "-frate");
+    if (frate > MAX_INT16 || frate > fe->sampling_rate || frate < 1) {
+        E_ERROR
+            ("Frame rate %d can not be bigger than sample rate %.02f\n",
+             frate, fe->sampling_rate);
+        return -1;
+    }
+
+    fe->frame_rate = (int16)frate;
     if (cmd_ln_boolean_r(config, "-dither")) {
         fe->dither = 1;
         fe->seed = cmd_ln_int32_r(config, "-seed");
@@ -195,7 +204,7 @@ fe_print_current(fe_t const *fe)
 fe_t *
 fe_init_auto()
 {
-    return fe_init_auto_r(cmd_ln_retain(cmd_ln_get()));
+    return fe_init_auto_r(cmd_ln_get());
 }
 
 fe_t *
@@ -207,7 +216,7 @@ fe_init_auto_r(cmd_ln_t *config)
     fe->refcount = 1;
 
     /* transfer params to front end */
-    if (fe_parse_general_params(config, fe) < 0) {
+    if (fe_parse_general_params(cmd_ln_retain(config), fe) < 0) {
         fe_free(fe);
         return NULL;
     }
@@ -221,6 +230,8 @@ fe_init_auto_r(cmd_ln_t *config)
     fe->prior = 0;
     fe->frame_counter = 0;
 
+    assert (fe->frame_shift > 1);
+
     if (fe->frame_size > (fe->fft_size)) {
         E_WARN
             ("Number of FFT points has to be a power of 2 higher than %d\n",
@@ -275,7 +286,7 @@ fe_get_args(void)
     return fe_args;
 }
 
-cmd_ln_t *
+const cmd_ln_t *
 fe_get_config(fe_t *fe)
 {
     return fe->config;
