@@ -49,15 +49,23 @@
 #pragma warning (disable: 4244)
 #endif
 
-#include "prim_type.h"
-#include "ckd_alloc.h"
-#include "byteorder.h"
-#include "fixpoint.h"
-#include "fe.h"
+/**
+ * Windows math.h does not contain M_PI
+ */
+#ifndef M_PI
+#define M_PI		3.14159265358979323846	/* pi */
+#endif // M_PI
+
+#include "sphinxbase/prim_type.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/byteorder.h"
+#include "sphinxbase/fixpoint.h"
+#include "sphinxbase/fe.h"
+#include "sphinxbase/genrand.h"
+#include "sphinxbase/err.h"
+
 #include "fe_internal.h"
 #include "fe_warp.h"
-#include "genrand.h"
-#include "err.h"
 
 /* Use extra precision for cosines, Hamming window, pre-emphasis
  * coefficient, twiddle factors. */
@@ -258,7 +266,7 @@ fe_log_add(fixed32 x, fixed32 y)
         d = (y - x) >> (DEFAULT_RADIX - 8);
         r = y;
     }
-    if (d > fe_logadd_table_size)
+    if (d > fe_logadd_table_size - 1)
         return r;
     else {
         r += ((fixed32)fe_logadd_table[d] << (DEFAULT_RADIX - 8));
@@ -394,7 +402,8 @@ fe_build_melfilters(melfb_t *mel_fb)
 
             hz = (mel_fb->spec_start[i] + j) * fftfreq;
             if (hz < freqs[0] || hz > freqs[2]) {
-                E_FATAL("WTF, %f < %f > %f\n", freqs[0], hz, freqs[2]);
+                E_FATAL("Failed to create filterbank, frequency range does not match. "
+                        "Sample rate %f, FFT size %d, lowerf %f < freq %f > upperf %f.\n", mel_fb->sampling_rate, mel_fb->fft_size, freqs[2], hz, freqs[0]);
             }
             loslope = (hz - freqs[0]) / (freqs[1] - freqs[0]);
             hislope = (freqs[2] - hz) / (freqs[2] - freqs[1]);
