@@ -126,16 +126,16 @@
 #pragma warning (disable: 4244 4996)
 #endif
 
-#include "fe.h"
-#include "feat.h"
-#include "bio.h"
-#include "pio.h"
-#include "cmn.h"
-#include "agc.h"
-#include "err.h"
-#include "ckd_alloc.h"
-#include "prim_type.h"
-#include "glist.h"
+#include "sphinxbase/fe.h"
+#include "sphinxbase/feat.h"
+#include "sphinxbase/bio.h"
+#include "sphinxbase/pio.h"
+#include "sphinxbase/cmn.h"
+#include "sphinxbase/agc.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/prim_type.h"
+#include "sphinxbase/glist.h"
 
 #define FEAT_VERSION	"1.0"
 #define FEAT_DCEP_WIN		2
@@ -352,169 +352,6 @@ feat_subvec_project(feat_t *fcb, mfcc_t ***inout_feat, uint32 nfr)
     }
 }
 
-/*
- * Read specified segment [sf-win..ef+win] of Sphinx-II format mfc file read and return
- * #frames read.  Return -1 if error.
- */
-int32
-feat_s2mfc_read(char *file, int32 win,
-                int32 sf, int32 ef,
-                mfcc_t ***out_mfc,
-                int32 maxfr,
-                int32 cepsize)
-{
-    FILE *fp;
-    int32 n_float32;
-    float32 *float_feat;
-    struct stat statbuf;
-    int32 i, n, byterev;
-    int32 start_pad, end_pad;
-    mfcc_t **mfc;
-
-    /* Initialize the output pointer to NULL, so that any attempts to
-       free() it if we fail before allocating it will not segfault! */
-    if (out_mfc)
-        *out_mfc = NULL;
-    E_INFO("Reading mfc file: '%s'[%d..%d]\n", file, sf, ef);
-    if (ef >= 0 && ef <= sf) {
-        E_ERROR("%s: End frame (%d) <= Start frame (%d)\n", file, ef, sf);
-        return -1;
-    }
-
-    /* Find filesize; HACK!! To get around intermittent NFS failures, use stat_retry */
-    if ((stat_retry(file, &statbuf) < 0)
-        || ((fp = fopen(file, "rb")) == NULL)) {
-        E_ERROR("stat_retry/fopen(%s) failed\n", file);
-        return -1;
-    }
-
-    /* Read #floats in header */
-    if (fread_retry(&n_float32, sizeof(int32), 1, fp) != 1) {
-        E_ERROR("%s: fread(#floats) failed\n", file);
-        fclose(fp);
-        return -1;
-    }
-
-    /* Check if n_float32 matches file size */
-    byterev = 0;
-    if ((int32) (n_float32 * sizeof(float32) + 4) != (int32) statbuf.st_size) { /* RAH, typecast both sides to remove compile warning */
-        n = n_float32;
-        SWAP_INT32(&n);
-
-        if ((int32) (n * sizeof(float32) + 4) != (int32) (statbuf.st_size)) {   /* RAH, typecast both sides to remove compile warning */
-            E_ERROR
-                ("%s: Header size field: %d(%08x); filesize: %d(%08x)\n",
-                 file, n_float32, n_float32, statbuf.st_size,
-                 statbuf.st_size);
-            fclose(fp);
-            return -1;
-        }
-
-        n_float32 = n;
-        byterev = 1;
-    }
-    if (n_float32 <= 0) {
-        E_ERROR("%s: Header size field (#floats) = %d\n", file, n_float32);
-        fclose(fp);
-        return -1;
-    }
-
-    /* Convert n to #frames of input */
-    n = n_float32 / cepsize;
-    if (n * cepsize != n_float32) {
-        E_ERROR("Header size field: %d; not multiple of %d\n", n_float32,
-                cepsize);
-        fclose(fp);
-        return -1;
-    }
-
-    /* Check start and end frames */
-    if (sf > 0) {
-        if (sf >= n) {
-            E_ERROR("%s: Start frame (%d) beyond file size (%d)\n", file,
-                    sf, n);
-            fclose(fp);
-            return -1;
-        }
-    }
-    if (ef < 0)
-        ef = n-1;
-    else if (ef >= n) {
-        E_WARN("%s: End frame (%d) beyond file size (%d), will truncate\n",
-               file, ef, n);
-        ef = n-1;
-    }
-
-    /* Add window to start and end frames */
-    sf -= win;
-    ef += win;
-    if (sf < 0) {
-        start_pad = -sf;
-        sf = 0;
-    }
-    else
-        start_pad = 0;
-    if (ef >= n) {
-        end_pad = ef - n + 1;
-        ef = n - 1;
-    }
-    else
-        end_pad = 0;
-
-    /* Limit n if indicated by [sf..ef] */
-    if ((ef - sf + 1) < n)
-        n = (ef - sf + 1);
-    if (maxfr > 0 && n + start_pad + end_pad > maxfr) {
-        E_ERROR("%s: Maximum output size(%d frames) < actual #frames(%d)\n",
-                file, maxfr, n + start_pad + end_pad);
-        fclose(fp);
-        return -1;
-    }
-
-    /* If no output buffer was supplied, then skip the actual data reading. */
-    if (out_mfc != NULL) {
-        /* Position at desired start frame and read actual MFC data */
-        mfc = (mfcc_t **)ckd_calloc_2d(n + start_pad + end_pad, cepsize, sizeof(mfcc_t));
-        if (sf > 0)
-            fseek(fp, sf * cepsize * sizeof(float32), SEEK_CUR);
-        n_float32 = n * cepsize;
-#ifdef FIXED_POINT
-        float_feat = ckd_calloc(n_float32, sizeof(float32));
-#else
-        float_feat = mfc[start_pad];
-#endif
-        if (fread_retry(float_feat, sizeof(float32), n_float32, fp) != n_float32) {
-            E_ERROR("%s: fread(%dx%d) (MFC data) failed\n", file, n, cepsize);
-            ckd_free_2d(mfc);
-            fclose(fp);
-            return -1;
-        }
-        if (byterev) {
-            for (i = 0; i < n_float32; i++) {
-                SWAP_FLOAT32(&float_feat[i]);
-            }
-        }
-#ifdef FIXED_POINT
-        for (i = 0; i < n_float32; ++i) {
-            mfc[start_pad][i] = FLOAT2MFCC(float_feat[i]);
-        }
-        ckd_free(float_feat);
-#endif
-
-        /* Replicate start and end frames if necessary. */
-        for (i = 0; i < start_pad; ++i)
-            memcpy(mfc[i], mfc[start_pad], cepsize * sizeof(mfcc_t));
-        for (i = 0; i < end_pad; ++i)
-            memcpy(mfc[start_pad + n + i], mfc[start_pad + n - 1],
-                   cepsize * sizeof(mfcc_t));
-
-        *out_mfc = mfc;
-    }
-
-    fclose(fp);
-    return n + start_pad + end_pad;
-}
-
 mfcc_t ***
 feat_array_alloc(feat_t * fcb, int32 nfr)
 {
@@ -548,6 +385,35 @@ feat_array_alloc(feat_t * fcb, int32 nfr)
     return feat;
 }
 
+mfcc_t ***
+feat_array_realloc(feat_t *fcb, mfcc_t ***old_feat, int32 ofr, int32 nfr)
+{
+    int32 i, k, cf;
+    mfcc_t*** new_feat;
+
+    assert(fcb);
+    assert(nfr > 0);
+    assert(ofr > 0);
+    assert(feat_dimension(fcb) > 0);
+
+    /* Make sure to use the dimensionality of the features *before*
+       LDA and subvector projection. */
+    k = 0;
+    for (i = 0; i < fcb->n_stream; ++i)
+        k += fcb->stream_len[i];
+    assert(k >= feat_dimension(fcb));
+    assert(k >= fcb->sv_dim);
+    
+    new_feat = feat_array_alloc(fcb, nfr);
+
+    cf = (nfr < ofr) ? nfr : ofr;
+    memcpy(new_feat[0][0], old_feat[0][0], cf * k * sizeof(mfcc_t));
+
+    feat_array_free(old_feat);
+    
+    return new_feat;
+}
+
 void
 feat_array_free(mfcc_t ***feat)
 {
@@ -683,7 +549,6 @@ feat_s3_cep(feat_t * fcb, mfcc_t ** mfc, mfcc_t ** feat)
     memcpy(feat[0], mfc[0], feat_cepsize(fcb) * sizeof(mfcc_t));
 }
 
-
 static void
 feat_s3_cep_dcep(feat_t * fcb, mfcc_t ** mfc, mfcc_t ** feat)
 {
@@ -861,7 +726,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         }
         fcb->cepsize = 13;
         fcb->n_stream = 4;
-        fcb->stream_len = (int32 *) ckd_calloc(4, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(4, sizeof(uint32));
         fcb->stream_len[0] = 12;
         fcb->stream_len[1] = 24;
         fcb->stream_len[2] = 3;
@@ -870,7 +735,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         fcb->window_size = 4;
         fcb->compute_feat = feat_s2_4x_cep2feat;
     }
-    else if (strcmp(type, "s3_1x39") == 0) {
+    else if ((strcmp(type, "s3_1x39") == 0) || (strcmp(type, "1s_12c_12d_3p_12dd") == 0)) {
         /* 1-stream cep/dcep/pow/ddcep (Hack!! hardwired constants below) */
         if (cepsize != 13) {
             E_ERROR("s2_4x features require cepsize == 13\n");
@@ -879,7 +744,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         }
         fcb->cepsize = 13;
         fcb->n_stream = 1;
-        fcb->stream_len = (int32 *) ckd_calloc(1, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
         fcb->stream_len[0] = 39;
         fcb->out_dim = 39;
         fcb->window_size = 3;
@@ -888,7 +753,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
     else if (strncmp(type, "1s_c_d_dd", 9) == 0) {
         fcb->cepsize = cepsize;
         fcb->n_stream = 1;
-        fcb->stream_len = (int32 *) ckd_calloc(1, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
         fcb->stream_len[0] = cepsize * 3;
         fcb->out_dim = cepsize * 3;
         fcb->window_size = FEAT_DCEP_WIN + 1; /* ddcep needs the extra 1 */
@@ -897,7 +762,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
     else if (strncmp(type, "1s_c_d_ld_dd", 12) == 0) {
         fcb->cepsize = cepsize;
         fcb->n_stream = 1;
-        fcb->stream_len = (int32 *) ckd_calloc(1, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
         fcb->stream_len[0] = cepsize * 4;
         fcb->out_dim = cepsize * 4;
         fcb->window_size = FEAT_DCEP_WIN * 2;
@@ -907,7 +772,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         /* 1-stream cep/dcep */
         fcb->cepsize = cepsize;
         fcb->n_stream = 1;
-        fcb->stream_len = (int32 *) ckd_calloc(1, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
         fcb->stream_len[0] = feat_cepsize(fcb) * 2;
         fcb->out_dim = fcb->stream_len[0];
         fcb->window_size = 2;
@@ -917,12 +782,26 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         /* 1-stream cep */
         fcb->cepsize = cepsize;
         fcb->n_stream = 1;
-        fcb->stream_len = (int32 *) ckd_calloc(1, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
         fcb->stream_len[0] = feat_cepsize(fcb);
         fcb->out_dim = fcb->stream_len[0];
         fcb->window_size = 0;
         fcb->compute_feat = feat_s3_cep;
     }
+    else if (strncmp(type, "1s_3c", 5) == 0 || strncmp(type, "1s_4c", 5) == 0) {
+	/* 1-stream cep with frames concatenated, so called cepwin features */
+        if (strncmp(type, "1s_3c", 5) == 0)
+            fcb->window_size = 3;
+        else
+    	    fcb->window_size = 4;
+
+        fcb->cepsize = cepsize;
+        fcb->n_stream = 1;
+        fcb->stream_len = (uint32 *) ckd_calloc(1, sizeof(uint32));
+        fcb->stream_len[0] = feat_cepsize(fcb) * (2 * fcb->window_size + 1);
+        fcb->out_dim = fcb->stream_len[0];
+        fcb->compute_feat = feat_copy;
+    }
     else {
         int32 i, l, k;
         char *strp;
@@ -950,7 +829,7 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         }
         k++;                    /* Presumably there are (#commas+1) streams */
         fcb->n_stream = k;
-        fcb->stream_len = (int32 *) ckd_calloc(k, sizeof(int32));
+        fcb->stream_len = (uint32 *) ckd_calloc(k, sizeof(uint32));
 
         /* Scan individual feature stream lengths */
         strp = mtype;
@@ -973,6 +852,8 @@ feat_init(char const *type, cmn_type_t cmn, int32 varnorm,
         }
         if (i != fcb->n_stream)
             E_FATAL("Bad feature type argument\n");
+        if (fcb->cepsize != cepsize)
+    	    E_FATAL("Bad feature type argument\n");
 
         /* Input is already the feature stream */
         fcb->compute_feat = feat_copy;
@@ -1087,8 +968,6 @@ feat_compute_utt(feat_t *fcb, mfcc_t **mfc, int32 nfr, int32 win, mfcc_t ***feat
     int32 i;
 
     cep_dump_dbg(fcb, mfc, nfr, "Incoming features (after padding)");
-    feat_cmn(fcb, mfc, nfr, 1, 1);
-    feat_agc(fcb, mfc, nfr, 1, 1);
 
     /* Create feature vectors */
     for (i = win; i < nfr - win; i++) {
@@ -1108,6 +987,184 @@ feat_compute_utt(feat_t *fcb, mfcc_t **mfc, int32 nfr, int32 win, mfcc_t ***feat
     }
 }
 
+
+/**
+ * Read Sphinx-II format mfc file (s2mfc = Sphinx-II format MFC data).
+ * If out_mfc is NULL, no actual reading will be done, and the number of 
+ * frames (plus padding) that would be read is returned.
+ * 
+ * It's important that normalization is done before padding because
+ * frames outside the data we are interested in shouldn't be taken
+ * into normalization stats.
+ *
+ * @return # frames read (plus padding) if successful, -1 if
+ * error (e.g., mfc array too small).  
+ */
+static int32
+feat_s2mfc_read_norm_pad(feat_t *fcb, char *file, int32 win,
+            		 int32 sf, int32 ef,
+            		 mfcc_t ***out_mfc,
+            		 int32 maxfr,
+            		 int32 cepsize)
+{
+    FILE *fp;
+    int32 n_float32;
+    float32 *float_feat;
+    struct stat statbuf;
+    int32 i, n, byterev;
+    int32 start_pad, end_pad;
+    mfcc_t **mfc;
+
+    /* Initialize the output pointer to NULL, so that any attempts to
+       free() it if we fail before allocating it will not segfault! */
+    if (out_mfc)
+        *out_mfc = NULL;
+    E_INFO("Reading mfc file: '%s'[%d..%d]\n", file, sf, ef);
+    if (ef >= 0 && ef <= sf) {
+        E_ERROR("%s: End frame (%d) <= Start frame (%d)\n", file, ef, sf);
+        return -1;
+    }
+
+    /* Find filesize; HACK!! To get around intermittent NFS failures, use stat_retry */
+    if ((stat_retry(file, &statbuf) < 0)
+        || ((fp = fopen(file, "rb")) == NULL)) {
+        E_ERROR("Failed to open file '%s' for reading: %s\n", file, strerror(errno));
+        return -1;
+    }
+
+    /* Read #floats in header */
+    if (fread_retry(&n_float32, sizeof(int32), 1, fp) != 1) {
+        E_ERROR("%s: fread(#floats) failed\n", file);
+        fclose(fp);
+        return -1;
+    }
+
+    /* Check if n_float32 matches file size */
+    byterev = 0;
+    if ((int32) (n_float32 * sizeof(float32) + 4) != (int32) statbuf.st_size) { /* RAH, typecast both sides to remove compile warning */
+        n = n_float32;
+        SWAP_INT32(&n);
+
+        if ((int32) (n * sizeof(float32) + 4) != (int32) (statbuf.st_size)) {   /* RAH, typecast both sides to remove compile warning */
+            E_ERROR
+                ("%s: Header size field: %d(%08x); filesize: %d(%08x)\n",
+                 file, n_float32, n_float32, statbuf.st_size,
+                 statbuf.st_size);
+            fclose(fp);
+            return -1;
+        }
+
+        n_float32 = n;
+        byterev = 1;
+    }
+    if (n_float32 <= 0) {
+        E_ERROR("%s: Header size field (#floats) = %d\n", file, n_float32);
+        fclose(fp);
+        return -1;
+    }
+
+    /* Convert n to #frames of input */
+    n = n_float32 / cepsize;
+    if (n * cepsize != n_float32) {
+        E_ERROR("Header size field: %d; not multiple of %d\n", n_float32,
+                cepsize);
+        fclose(fp);
+        return -1;
+    }
+
+    /* Check start and end frames */
+    if (sf > 0) {
+        if (sf >= n) {
+            E_ERROR("%s: Start frame (%d) beyond file size (%d)\n", file,
+                    sf, n);
+            fclose(fp);
+            return -1;
+        }
+    }
+    if (ef < 0)
+        ef = n-1;
+    else if (ef >= n) {
+        E_WARN("%s: End frame (%d) beyond file size (%d), will truncate\n",
+               file, ef, n);
+        ef = n-1;
+    }
+
+    /* Add window to start and end frames */
+    sf -= win;
+    ef += win;
+    if (sf < 0) {
+        start_pad = -sf;
+        sf = 0;
+    }
+    else
+        start_pad = 0;
+    if (ef >= n) {
+        end_pad = ef - n + 1;
+        ef = n - 1;
+    }
+    else
+        end_pad = 0;
+
+    /* Limit n if indicated by [sf..ef] */
+    if ((ef - sf + 1) < n)
+        n = (ef - sf + 1);
+    if (maxfr > 0 && n + start_pad + end_pad > maxfr) {
+        E_ERROR("%s: Maximum output size(%d frames) < actual #frames(%d)\n",
+                file, maxfr, n + start_pad + end_pad);
+        fclose(fp);
+        return -1;
+    }
+
+    /* If no output buffer was supplied, then skip the actual data reading. */
+    if (out_mfc != NULL) {
+        /* Position at desired start frame and read actual MFC data */
+        mfc = (mfcc_t **)ckd_calloc_2d(n + start_pad + end_pad, cepsize, sizeof(mfcc_t));
+        if (sf > 0)
+            fseek(fp, sf * cepsize * sizeof(float32), SEEK_CUR);
+        n_float32 = n * cepsize;
+#ifdef FIXED_POINT
+        float_feat = ckd_calloc(n_float32, sizeof(float32));
+#else
+        float_feat = mfc[start_pad];
+#endif
+        if (fread_retry(float_feat, sizeof(float32), n_float32, fp) != n_float32) {
+            E_ERROR("%s: fread(%dx%d) (MFC data) failed\n", file, n, cepsize);
+            ckd_free_2d(mfc);
+            fclose(fp);
+            return -1;
+        }
+        if (byterev) {
+            for (i = 0; i < n_float32; i++) {
+                SWAP_FLOAT32(&float_feat[i]);
+            }
+        }
+#ifdef FIXED_POINT
+        for (i = 0; i < n_float32; ++i) {
+            mfc[start_pad][i] = FLOAT2MFCC(float_feat[i]);
+        }
+        ckd_free(float_feat);
+#endif
+
+        /* Normalize */
+        feat_cmn(fcb, mfc + start_pad, n, 1, 1);
+        feat_agc(fcb, mfc + start_pad, n, 1, 1);
+
+        /* Replicate start and end frames if necessary. */
+        for (i = 0; i < start_pad; ++i)
+            memcpy(mfc[i], mfc[start_pad], cepsize * sizeof(mfcc_t));
+        for (i = 0; i < end_pad; ++i)
+            memcpy(mfc[start_pad + n + i], mfc[start_pad + n - 1],
+                   cepsize * sizeof(mfcc_t));
+
+        *out_mfc = mfc;
+    }
+
+    fclose(fp);
+    return n + start_pad + end_pad;
+}
+
+
+
 int32
 feat_s2mfc2feat(feat_t * fcb, const char *file, const char *dir, const char *cepext,
                 int32 sf, int32 ef, mfcc_t *** feat, int32 maxfr)
@@ -1189,19 +1246,21 @@ feat_s2mfc2feat(feat_t * fcb, const char *file, const char *dir, const char *cep
 
     if (feat != NULL) {
         /* Read mfc file including window or padding if necessary. */
-        nfr = feat_s2mfc_read(path, win, sf, ef, &mfc, maxfr, fcb->cepsize);
+        nfr = feat_s2mfc_read_norm_pad(fcb, path, win, sf, ef, &mfc, maxfr, fcb->cepsize);
         ckd_free(path);
         if (nfr < 0) {
             ckd_free_2d((void **) mfc);
             return -1;
         }
+
         /* Actually compute the features */
         feat_compute_utt(fcb, mfc, nfr, win, feat);
+        
         ckd_free_2d((void **) mfc);
     }
     else {
         /* Just calculate the number of frames we would need. */
-        nfr = feat_s2mfc_read(path, win, sf, ef, NULL, maxfr, fcb->cepsize);
+        nfr = feat_s2mfc_read_norm_pad(fcb, path, win, sf, ef, NULL, maxfr, fcb->cepsize);
         ckd_free(path);
         if (nfr < 0)
             return nfr;
@@ -1226,6 +1285,12 @@ feat_s2mfc2feat_block_utt(feat_t * fcb, mfcc_t ** uttcep,
      * the frame pointers, which they do)  */
     cepbuf = ckd_calloc(nfr + win * 2, sizeof(mfcc_t *));
     memcpy(cepbuf + win, uttcep, nfr * sizeof(mfcc_t *));
+
+    /* Do normalization before we interpolate on the boundary */    
+    feat_cmn(fcb, cepbuf + win, nfr, 1, 1);
+    feat_agc(fcb, cepbuf + win, nfr, 1, 1);
+
+    /* Now interpolate */    
     for (i = 0; i < win; ++i) {
         cepbuf[i] = fcb->cepbuf[i];
         memcpy(cepbuf[i], uttcep[0], cepsize * sizeof(mfcc_t));
