@@ -39,21 +39,39 @@
 #include <windows.h>
 #else
 #include <time.h>
-#endif /* _WIN32_WCE */
+#endif                          /* _WIN32_WCE */
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 
 /* SphinxBase headers. */
-#include "err.h"
-#include "pio.h"
-#include "ckd_alloc.h"
-#include "prim_type.h"
-#include "strfuncs.h"
-#include "hash_table.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/pio.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/prim_type.h"
+#include "sphinxbase/strfuncs.h"
+#include "sphinxbase/hash_table.h"
+#include "sphinxbase/fsg_model.h"
 
-#include "fsg_model.h"
+/**
+ * Adjacency list (opaque) for a state in an FSG.
+ *
+ * Actually we use hash tables so that random access is a bit faster.
+ * Plus it allows us to make the lookup code a bit less ugly.
+ */
 
+struct trans_list_s {
+    hash_table_t *null_trans;   /* Null transitions keyed by state. */
+    hash_table_t *trans;        /* Lists of non-null transitions keyed by state. */
+};
+
+/**
+ * Implementation of arc iterator.
+ */
+struct fsg_arciter_s {
+    hash_iter_t *itor, *null_itor;
+    gnode_t *gn;
+};
 
 #define FSG_MODEL_BEGIN_DECL		"FSG_BEGIN"
 #define FSG_MODEL_END_DECL		"FSG_END"
@@ -83,11 +101,11 @@ nextline_str2words(FILE * fp, int32 * lineno,
         (*lineno)++;
 
         if ((*lineptr)[0] == FSG_MODEL_COMMENT_CHAR)
-            continue; /* Skip comment lines */
-        
+            continue;           /* Skip comment lines */
+
         n = str2words(*lineptr, NULL, 0);
         if (n == 0)
-            continue; /* Skip blank lines */
+            continue;           /* Skip blank lines */
 
         /* Abuse of realloc(), but this doesn't have to be fast. */
         if (*wordptr == NULL)
@@ -100,15 +118,18 @@ nextline_str2words(FILE * fp, int32 * lineno,
 
 void
 fsg_model_trans_add(fsg_model_t * fsg,
-                   int32 from, int32 to, int32 logp, int32 wid)
+                    int32 from, int32 to, int32 logp, int32 wid)
 {
     fsg_link_t *link;
+    glist_t gl;
     gnode_t *gn;
 
-    /* Check for duplicate link (i.e., link already exists with label=wid) */
-    for (gn = fsg->trans[from][to]; gn; gn = gnode_next(gn)) {
-        link = (fsg_link_t *) gnode_ptr(gn);
+    if (fsg->trans[from].trans == NULL)
+        fsg->trans[from].trans = hash_table_new(5, HASH_CASE_YES);
 
+    /* Check for duplicate link (i.e., link already exists with label=wid) */
+    for (gn = gl = fsg_model_trans(fsg, from, to); gn; gn = gnode_next(gn)) {
+        link = (fsg_link_t *) gnode_ptr(gn);
         if (link->wid == wid) {
             if (link->logs2prob < logp)
                 link->logs2prob = logp;
@@ -123,14 +144,18 @@ fsg_model_trans_add(fsg_model_t * fsg,
     link->logs2prob = logp;
     link->wid = wid;
 
-    fsg->trans[from][to] =
-        glist_add_ptr(fsg->trans[from][to], (void *) link);
+    /* Add it to the list of transitions and update the hash table */
+    gl = glist_add_ptr(gl, (void *) link);
+    hash_table_replace_bkey(fsg->trans[from].trans,
+                            (char const *) &link->to_state,
+                            sizeof(link->to_state), gl);
 }
 
 int32
-fsg_model_null_trans_add(fsg_model_t * fsg, int32 from, int32 to, int32 logp)
+fsg_model_tag_trans_add(fsg_model_t * fsg, int32 from, int32 to,
+                        int32 logp, int32 wid)
 {
-    fsg_link_t *link;
+    fsg_link_t *link, *link2;
 
     /* Check for transition probability */
     if (logp > 0) {
@@ -142,10 +167,12 @@ fsg_model_null_trans_add(fsg_model_t * fsg, int32 from, int32 to, int32 logp)
     if (from == to)
         return -1;
 
+    if (fsg->trans[from].null_trans == NULL)
+        fsg->trans[from].null_trans = hash_table_new(5, HASH_CASE_YES);
+
     /* Check for a duplicate link; if found, keep the higher prob */
-    link = fsg->null_trans[from][to];
+    link = fsg_model_null_trans(fsg, from, to);
     if (link) {
-        assert(link->wid < 0);
         if (link->logs2prob < logp) {
             link->logs2prob = logp;
             return 0;
@@ -161,15 +188,26 @@ fsg_model_null_trans_add(fsg_model_t * fsg, int32 from, int32 to, int32 logp)
     link->logs2prob = logp;
     link->wid = -1;
 
-    fsg->null_trans[from][to] = link;
+    link2 = (fsg_link_t *)
+        hash_table_enter_bkey(fsg->trans[from].null_trans,
+                              (char const *) &link->to_state,
+                              sizeof(link->to_state), link);
+    assert(link == link2);
 
     return 1;
 }
 
+int32
+fsg_model_null_trans_add(fsg_model_t * fsg, int32 from, int32 to,
+                         int32 logp)
+{
+    return fsg_model_tag_trans_add(fsg, from, to, logp, -1);
+}
+
 glist_t
 fsg_model_null_trans_closure(fsg_model_t * fsg, glist_t nulls)
 {
-    gnode_t *gn1, *gn2;
+    gnode_t *gn1;
     int updated;
     fsg_link_t *tl1, *tl2;
     int32 k, n;
@@ -177,12 +215,13 @@ fsg_model_null_trans_closure(fsg_model_t * fsg, glist_t nulls)
     E_INFO("Computing transitive closure for null transitions\n");
 
     if (nulls == NULL) {
+        fsg_link_t *null;
         int i, j;
-        
+
         for (i = 0; i < fsg->n_state; ++i) {
             for (j = 0; j < fsg->n_state; ++j) {
-                if (fsg->null_trans[i][j])
-                    nulls = glist_add_ptr(nulls, fsg->null_trans[i][j]);
+                if ((null = fsg_model_null_trans(fsg, i, j)))
+                    nulls = glist_add_ptr(nulls, null);
             }
         }
     }
@@ -196,42 +235,139 @@ fsg_model_null_trans_closure(fsg_model_t * fsg, glist_t nulls)
         updated = FALSE;
 
         for (gn1 = nulls; gn1; gn1 = gnode_next(gn1)) {
+            hash_iter_t *itor;
+
             tl1 = (fsg_link_t *) gnode_ptr(gn1);
             assert(tl1->wid < 0);
 
-            for (gn2 = nulls; gn2; gn2 = gnode_next(gn2)) {
-                tl2 = (fsg_link_t *) gnode_ptr(gn2);
+            if (fsg->trans[tl1->to_state].null_trans == NULL)
+                continue;
 
-                if (tl1->to_state == tl2->from_state) {
-                    k = fsg_model_null_trans_add(fsg,
-                                                tl1->from_state,
-                                                tl2->to_state,
-                                                tl1->logs2prob +
-                                                tl2->logs2prob);
-                    if (k >= 0) {
-                        updated = TRUE;
-                        if (k > 0) {
-                            nulls =
-                                glist_add_ptr(nulls,
-                                              (void *) fsg->
-                                              null_trans[tl1->
-                                                         from_state][tl2->
-                                                                     to_state]);
-                            n++;
-                        }
+            for (itor = hash_table_iter(fsg->trans[tl1->to_state].null_trans);
+                 itor; itor = hash_table_iter_next(itor)) {
+
+                tl2 = (fsg_link_t *) hash_entry_val(itor->ent);
+
+                k = fsg_model_null_trans_add(fsg,
+                                             tl1->from_state,
+                                             tl2->to_state,
+                                             tl1->logs2prob +
+                                             tl2->logs2prob);
+                if (k >= 0) {
+                    updated = TRUE;
+                    if (k > 0) {
+                        nulls = glist_add_ptr(nulls, (void *)
+                                              fsg_model_null_trans
+                                              (fsg, tl1->from_state,
+                                               tl2->to_state));
+                        n++;
                     }
                 }
             }
         }
     } while (updated);
-
+    
     E_INFO("%d null transitions added\n", n);
 
     return nulls;
 }
 
+glist_t
+fsg_model_trans(fsg_model_t * fsg, int32 i, int32 j)
+{
+    void *val;
+
+    if (fsg->trans[i].trans == NULL)
+        return NULL;
+    if (hash_table_lookup_bkey(fsg->trans[i].trans, (char const *) &j,
+                               sizeof(j), &val) < 0)
+        return NULL;
+    return (glist_t) val;
+}
+
+fsg_link_t *
+fsg_model_null_trans(fsg_model_t * fsg, int32 i, int32 j)
+{
+    void *val;
+
+    if (fsg->trans[i].null_trans == NULL)
+        return NULL;
+    if (hash_table_lookup_bkey(fsg->trans[i].null_trans, (char const *) &j,
+                               sizeof(j), &val) < 0)
+        return NULL;
+    return (fsg_link_t *) val;
+}
+
+fsg_arciter_t *
+fsg_model_arcs(fsg_model_t * fsg, int32 i)
+{
+    fsg_arciter_t *itor;
+
+    if (fsg->trans[i].trans == NULL && fsg->trans[i].null_trans == NULL)
+        return NULL;
+    itor = ckd_calloc(1, sizeof(*itor));
+    if (fsg->trans[i].null_trans)
+        itor->null_itor = hash_table_iter(fsg->trans[i].null_trans);
+    if (fsg->trans[i].trans)
+        itor->itor = hash_table_iter(fsg->trans[i].trans);
+    if (itor->itor != NULL)
+        itor->gn = hash_entry_val(itor->itor->ent);
+    return itor;
+}
+
+fsg_link_t *
+fsg_arciter_get(fsg_arciter_t * itor)
+{
+    /* Iterate over non-null arcs first. */
+    if (itor->gn)
+        return (fsg_link_t *) gnode_ptr(itor->gn);
+    else if (itor->null_itor)
+        return (fsg_link_t *) hash_entry_val(itor->null_itor->ent);
+    else
+        return NULL;
+}
+
+fsg_arciter_t *
+fsg_arciter_next(fsg_arciter_t * itor)
+{
+    /* Iterate over non-null arcs first. */
+    if (itor->gn) {
+        itor->gn = gnode_next(itor->gn);
+        /* Move to the next destination arc. */
+        if (itor->gn == NULL) {
+            itor->itor = hash_table_iter_next(itor->itor);
+            if (itor->itor != NULL)
+                itor->gn = hash_entry_val(itor->itor->ent);
+            else if (itor->null_itor == NULL)
+                goto stop_iteration;
+        }
+    }
+    else {
+        if (itor->null_itor == NULL)
+            goto stop_iteration;
+        itor->null_itor = hash_table_iter_next(itor->null_itor);
+        if (itor->null_itor == NULL)
+            goto stop_iteration;
+    }
+    return itor;
+  stop_iteration:
+    fsg_arciter_free(itor);
+    return NULL;
+
+}
+
+void
+fsg_arciter_free(fsg_arciter_t * itor)
+{
+    if (itor == NULL)
+        return;
+    hash_table_iter_free(itor->null_itor);
+    hash_table_iter_free(itor->itor);
+    ckd_free(itor);
+}
+
 int
-fsg_model_word_id(fsg_model_t *fsg, char const *word)
+fsg_model_word_id(fsg_model_t * fsg, char const *word)
 {
     int wid;
 
@@ -247,9 +383,9 @@ fsg_model_word_id(fsg_model_t *fsg, char const *word)
 }
 
 int
-fsg_model_word_add(fsg_model_t *fsg, char const *word)
+fsg_model_word_add(fsg_model_t * fsg, char const *word)
 {
-    int wid;
+    int wid, old_size;
 
     /* Search for an existing word matching this. */
     wid = fsg_model_word_id(fsg, word);
@@ -257,13 +393,17 @@ fsg_model_word_add(fsg_model_t *fsg, char const *word)
     if (wid == -1) {
         wid = fsg->n_word;
         if (fsg->n_word == fsg->n_word_alloc) {
+    	    old_size = fsg->n_word_alloc;
             fsg->n_word_alloc += 10;
             fsg->vocab = ckd_realloc(fsg->vocab,
-                                     fsg->n_word_alloc * sizeof(*fsg->vocab));
+                                     fsg->n_word_alloc *
+                                     sizeof(*fsg->vocab));
             if (fsg->silwords)
-                fsg->silwords = bitvec_realloc(fsg->silwords, fsg->n_word_alloc);
+                fsg->silwords =
+                    bitvec_realloc(fsg->silwords, old_size, fsg->n_word_alloc);
             if (fsg->altwords)
-                fsg->altwords = bitvec_realloc(fsg->altwords, fsg->n_word_alloc);
+                fsg->altwords =
+                    bitvec_realloc(fsg->altwords, old_size, fsg->n_word_alloc);
         }
         ++fsg->n_word;
         fsg->vocab[wid] = ckd_salloc(word);
@@ -306,7 +446,7 @@ int
 fsg_model_add_alt(fsg_model_t * fsg, char const *baseword,
                   char const *altword)
 {
-    int i, j, basewid, altwid;
+    int i, basewid, altwid;
     int ntrans;
 
     /* FIXME: This will get slow, eventually... */
@@ -322,18 +462,22 @@ fsg_model_add_alt(fsg_model_t * fsg, char const *baseword,
         fsg->altwords = bitvec_alloc(fsg->n_word_alloc);
     bitvec_set(fsg->altwords, altwid);
 
-    E_INFO("Adding alternate word transitions (%s,%s) to FSG\n",
-           baseword, altword);
+    E_DEBUG(2, ("Adding alternate word transitions (%s,%s) to FSG\n",
+                baseword, altword));
 
     /* Look for all transitions involving baseword and duplicate them. */
     /* FIXME: This will also get slow, eventually... */
     ntrans = 0;
     for (i = 0; i < fsg->n_state; ++i) {
-        for (j = 0; j < fsg->n_state; ++j) {
+        hash_iter_t *itor;
+        if (fsg->trans[i].trans == NULL)
+            continue;
+        for (itor = hash_table_iter(fsg->trans[i].trans); itor;
+             itor = hash_table_iter_next(itor)) {
             glist_t trans;
             gnode_t *gn;
 
-            trans = fsg->trans[i][j];
+            trans = hash_entry_val(itor->ent);
             for (gn = trans; gn; gn = gnode_next(gn)) {
                 fsg_link_t *fl = gnode_ptr(gn);
                 if (fl->wid == basewid) {
@@ -341,27 +485,27 @@ fsg_model_add_alt(fsg_model_t * fsg, char const *baseword,
 
                     /* Create transition object */
                     link = listelem_malloc(fsg->link_alloc);
-                    link->from_state = i;
-                    link->to_state = j;
-                    link->logs2prob = fl->logs2prob; /* FIXME!!!??? */
+                    link->from_state = fl->from_state;
+                    link->to_state = fl->to_state;
+                    link->logs2prob = fl->logs2prob;    /* FIXME!!!??? */
                     link->wid = altwid;
 
-                    trans =
-                        glist_add_ptr(trans, (void *) link);
+                    trans = glist_add_ptr(trans, (void *) link);
                     ++ntrans;
                 }
             }
-            fsg->trans[i][j] = trans;
+            hash_entry_val(itor->ent) = trans;
         }
     }
 
-    E_INFO("Added %d alternate word transitions\n", ntrans);
+    E_DEBUG(2, ("Added %d alternate word transitions\n", ntrans));
     return ntrans;
 }
 
 
 fsg_model_t *
-fsg_model_init(char const *name, logmath_t *lmath, float32 lw, int32 n_state)
+fsg_model_init(char const *name, logmath_t * lmath, float32 lw,
+               int32 n_state)
 {
     fsg_model_t *fsg;
 
@@ -374,17 +518,13 @@ fsg_model_init(char const *name, logmath_t *lmath, float32 lw, int32 n_state)
     fsg->n_state = n_state;
     fsg->lw = lw;
 
-    /* Allocate non-epsilon transition matrix array */
-    fsg->trans = ckd_calloc_2d(fsg->n_state, fsg->n_state,
-                               sizeof(glist_t));
-    /* Allocate epsilon transition matrix array */
-    fsg->null_trans = ckd_calloc_2d(fsg->n_state, fsg->n_state,
-                                    sizeof(fsg_link_t *));
+    fsg->trans = ckd_calloc(fsg->n_state, sizeof(*fsg->trans));
+
     return fsg;
 }
 
 fsg_model_t *
-fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
+fsg_model_read(FILE * fp, logmath_t * lmath, float32 lw)
 {
     fsg_model_t *fsg;
     hash_table_t *vocab;
@@ -429,8 +569,9 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
      */
     if (n == 2) {
         fsgname = ckd_salloc(wordptr[1]);
-    } else {
-        E_WARN ("FSG name is missing\n");
+    }
+    else {
+        E_WARN("FSG name is missing\n");
         fsgname = ckd_salloc("unknown");
     }
 
@@ -526,11 +667,13 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
             goto parse_error;
         }
 
-        tprob = (int32)(logmath_log(lmath, p) * fsg->lw);
+        tprob = (int32) (logmath_log(lmath, p) * fsg->lw);
         /* Add word to "dictionary". */
         if (n > 4) {
             if (hash_table_lookup_int32(vocab, wordptr[4], &wid) < 0) {
-                (void)hash_table_enter_int32(vocab, ckd_salloc(wordptr[4]), lastwid);
+                (void) hash_table_enter_int32(vocab,
+                                              ckd_salloc(wordptr[4]),
+                                              lastwid);
                 wid = lastwid;
                 ++lastwid;
             }
@@ -540,7 +683,8 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
         else {
             if (fsg_model_null_trans_add(fsg, i, j, tprob) == 1) {
                 ++n_null_trans;
-                nulls = glist_add_ptr(nulls, fsg->null_trans[i][j]);
+                nulls =
+                    glist_add_ptr(nulls, fsg_model_null_trans(fsg, i, j));
             }
         }
     }
@@ -548,28 +692,32 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
     E_INFO("FSG: %d states, %d unique words, %d transitions (%d null)\n",
            fsg->n_state, hash_table_inuse(vocab), n_trans, n_null_trans);
 
+
+    /* Now create a string table from the "dictionary" */
+    fsg->n_word = hash_table_inuse(vocab);
+    fsg->n_word_alloc = fsg->n_word + 10;       /* Pad it a bit. */
+    fsg->vocab = ckd_calloc(fsg->n_word_alloc, sizeof(*fsg->vocab));
+    for (itor = hash_table_iter(vocab); itor;
+         itor = hash_table_iter_next(itor)) {
+        char const *word = hash_entry_key(itor->ent);
+        int32 wid = (int32) (long) hash_entry_val(itor->ent);
+        fsg->vocab[wid] = (char *) word;
+    }
+    hash_table_free(vocab);
+
     /* Do transitive closure on null transitions */
     nulls = fsg_model_null_trans_closure(fsg, nulls);
     glist_free(nulls);
 
-    /* Now create a string table from the "dictionary" */
-    fsg->n_word = hash_table_inuse(vocab);
-    fsg->n_word_alloc = fsg->n_word + 10; /* Pad it a bit. */
-    fsg->vocab = ckd_calloc(fsg->n_word_alloc, sizeof(*fsg->vocab));
-    for (itor = hash_table_iter(vocab); itor; itor = hash_table_iter_next(itor)) {
-        char const *word = hash_entry_key(itor->ent);
-        int32 wid = (int32)(long)hash_entry_val(itor->ent);
-        fsg->vocab[wid] = (char *)word;
-    }
-    hash_table_free(vocab);
     ckd_free(lineptr);
     ckd_free(wordptr);
 
     return fsg;
 
   parse_error:
-    for (itor = hash_table_iter(vocab); itor; itor = hash_table_iter_next(itor))
-        ckd_free((char *)hash_entry_key(itor->ent));
+    for (itor = hash_table_iter(vocab); itor;
+         itor = hash_table_iter_next(itor))
+        ckd_free((char *) hash_entry_key(itor->ent));
     glist_free(nulls);
     hash_table_free(vocab);
     ckd_free(fsgname);
@@ -581,13 +729,13 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
 
 
 fsg_model_t *
-fsg_model_readfile(const char *file, logmath_t *lmath, float32 lw)
+fsg_model_readfile(const char *file, logmath_t * lmath, float32 lw)
 {
     FILE *fp;
     fsg_model_t *fsg;
 
     if ((fp = fopen(file, "r")) == NULL) {
-        E_ERROR("fopen(%s,r) failed\n", file);
+        E_ERROR_SYSTEM("Failed to open FSG file '%s' for reading", file);
         return NULL;
     }
     fsg = fsg_model_read(fp, lmath, lw);
@@ -596,16 +744,35 @@ fsg_model_readfile(const char *file, logmath_t *lmath, float32 lw)
 }
 
 fsg_model_t *
-fsg_model_retain(fsg_model_t *fsg)
+fsg_model_retain(fsg_model_t * fsg)
 {
     ++fsg->refcount;
     return fsg;
 }
 
+static void
+trans_list_free(fsg_model_t * fsg, int32 i)
+{
+    hash_iter_t *itor;
+
+    /* FIXME (maybe): FSG links will all get freed when we call
+     * listelem_alloc_free() so don't bother freeing them explicitly
+     * here. */
+    if (fsg->trans[i].trans) {
+        for (itor = hash_table_iter(fsg->trans[i].trans);
+             itor; itor = hash_table_iter_next(itor)) {
+            glist_t gl = (glist_t) hash_entry_val(itor->ent);
+            glist_free(gl);
+        }
+    }
+    hash_table_free(fsg->trans[i].trans);
+    hash_table_free(fsg->trans[i].null_trans);
+}
+
 int
 fsg_model_free(fsg_model_t * fsg)
 {
-    int i, j;
+    int i;
 
     if (fsg == NULL)
         return 0;
@@ -616,14 +783,12 @@ fsg_model_free(fsg_model_t * fsg)
     for (i = 0; i < fsg->n_word; ++i)
         ckd_free(fsg->vocab[i]);
     for (i = 0; i < fsg->n_state; ++i)
-        for (j = 0; j < fsg->n_state; ++j)
-            glist_free(fsg->trans[i][j]);
+        trans_list_free(fsg, i);
+    ckd_free(fsg->trans);
     ckd_free(fsg->vocab);
     listelem_alloc_free(fsg->link_alloc);
     bitvec_free(fsg->silwords);
     bitvec_free(fsg->altwords);
-    ckd_free_2d(fsg->trans);
-    ckd_free_2d(fsg->null_trans);
     ckd_free(fsg->name);
     ckd_free(fsg);
     return 0;
@@ -633,35 +798,26 @@ fsg_model_free(fsg_model_t * fsg)
 void
 fsg_model_write(fsg_model_t * fsg, FILE * fp)
 {
-    int32 i, j;
-    gnode_t *gn;
-    fsg_link_t *tl;
+    int32 i;
 
-    fprintf(fp, "%s %s\n", FSG_MODEL_BEGIN_DECL, fsg->name ? fsg->name : "");
+    fprintf(fp, "%s %s\n", FSG_MODEL_BEGIN_DECL,
+            fsg->name ? fsg->name : "");
     fprintf(fp, "%s %d\n", FSG_MODEL_NUM_STATES_DECL, fsg->n_state);
     fprintf(fp, "%s %d\n", FSG_MODEL_START_STATE_DECL, fsg->start_state);
     fprintf(fp, "%s %d\n", FSG_MODEL_FINAL_STATE_DECL, fsg->final_state);
 
     for (i = 0; i < fsg->n_state; i++) {
-        for (j = 0; j < fsg->n_state; j++) {
-            /* Print non-null transitions */
-            for (gn = fsg->trans[i][j]; gn; gn = gnode_next(gn)) {
-                tl = (fsg_link_t *) gnode_ptr(gn);
+        fsg_arciter_t *itor;
 
-                fprintf(fp, "%s %d %d %f %s\n", FSG_MODEL_TRANSITION_DECL,
-                        tl->from_state, tl->to_state,
-                        logmath_exp(fsg->lmath, (int32)(tl->logs2prob / fsg->lw)),
-                        (tl->wid < 0) ? "" : fsg_model_word_str(fsg, tl->wid));
-            }
+        for (itor = fsg_model_arcs(fsg, i); itor;
+             itor = fsg_arciter_next(itor)) {
+            fsg_link_t *tl = fsg_arciter_get(itor);
 
-            /* Print null transitions */
-            tl = fsg->null_trans[i][j];
-            if (tl) {
-                fprintf(fp, "%s %d %d %f\n",
-                        FSG_MODEL_TRANSITION_DECL,
-                        tl->from_state, tl->to_state,
-                        logmath_exp(fsg->lmath, (int32)(tl->logs2prob / fsg->lw)));
-            }
+            fprintf(fp, "%s %d %d %f %s\n", FSG_MODEL_TRANSITION_DECL,
+                    tl->from_state, tl->to_state,
+                    logmath_exp(fsg->lmath,
+                                (int32) (tl->logs2prob / fsg->lw)),
+                    (tl->wid < 0) ? "" : fsg_model_word_str(fsg, tl->wid));
         }
     }
 
@@ -671,7 +827,7 @@ fsg_model_write(fsg_model_t * fsg, FILE * fp)
 }
 
 void
-fsg_model_writefile(fsg_model_t *fsg, char const *file)
+fsg_model_writefile(fsg_model_t * fsg, char const *file)
 {
     FILE *fp;
 
@@ -680,7 +836,7 @@ fsg_model_writefile(fsg_model_t *fsg, char const *file)
     E_INFO("Writing FSG file '%s'\n", file);
 
     if ((fp = fopen(file, "w")) == NULL) {
-        E_ERROR("fopen(%s,r) failed\n", file);
+        E_ERROR_SYSTEM("Failed to open FSG file '%s' for reading", file);
         return;
     }
 
@@ -690,31 +846,17 @@ fsg_model_writefile(fsg_model_t *fsg, char const *file)
 }
 
 static void
-fsg_model_write_fsm_trans(fsg_model_t *fsg, int i, FILE *fp)
+fsg_model_write_fsm_trans(fsg_model_t * fsg, int i, FILE * fp)
 {
-    int j;
+    fsg_arciter_t *itor;
 
-    for (j = 0; j < fsg->n_state; j++) {
-        gnode_t *gn;
-        fsg_link_t *tl;
-
-        /* Print non-null transitions */
-        for (gn = fsg->trans[i][j]; gn; gn = gnode_next(gn)) {
-            tl = (fsg_link_t *) gnode_ptr(gn);
-
-            fprintf(fp, "%d %d %s %f\n",
-                    tl->from_state, tl->to_state,
-                    (tl->wid < 0) ? "<eps>" : fsg_model_word_str(fsg, tl->wid),
-                    -logmath_log_to_ln(fsg->lmath, tl->logs2prob / fsg->lw));
-        }
-
-        /* Print null transitions */
-        tl = fsg->null_trans[i][j];
-        if (tl) {
-            fprintf(fp, "%d %d <eps> %f\n",
-                    tl->from_state, tl->to_state,
-                    -logmath_log_to_ln(fsg->lmath, tl->logs2prob / fsg->lw));
-        }
+    for (itor = fsg_model_arcs(fsg, i); itor;
+         itor = fsg_arciter_next(itor)) {
+        fsg_link_t *tl = fsg_arciter_get(itor);
+        fprintf(fp, "%d %d %s %f\n",
+                tl->from_state, tl->to_state,
+                (tl->wid < 0) ? "<eps>" : fsg_model_word_str(fsg, tl->wid),
+                -logmath_log_to_ln(fsg->lmath, tl->logs2prob / fsg->lw));
     }
 }
 
@@ -740,7 +882,7 @@ fsg_model_write_fsm(fsg_model_t * fsg, FILE * fp)
 }
 
 void
-fsg_model_writefile_fsm(fsg_model_t *fsg, char const *file)
+fsg_model_writefile_fsm(fsg_model_t * fsg, char const *file)
 {
     FILE *fp;
 
@@ -749,7 +891,7 @@ fsg_model_writefile_fsm(fsg_model_t *fsg, char const *file)
     E_INFO("Writing FSM file '%s'\n", file);
 
     if ((fp = fopen(file, "w")) == NULL) {
-        E_ERROR("fopen(%s,r) failed\n", file);
+        E_ERROR_SYSTEM("Failed to open fsm file '%s' for writing", file);
         return;
     }
 
@@ -759,7 +901,7 @@ fsg_model_writefile_fsm(fsg_model_t *fsg, char const *file)
 }
 
 void
-fsg_model_write_symtab(fsg_model_t *fsg, FILE *file)
+fsg_model_write_symtab(fsg_model_t * fsg, FILE * file)
 {
     int i;
 
@@ -771,7 +913,7 @@ fsg_model_write_symtab(fsg_model_t *fsg, FILE *file)
 }
 
 void
-fsg_model_writefile_symtab(fsg_model_t *fsg, char const *file)
+fsg_model_writefile_symtab(fsg_model_t * fsg, char const *file)
 {
     FILE *fp;
 
@@ -780,7 +922,7 @@ fsg_model_writefile_symtab(fsg_model_t *fsg, char const *file)
     E_INFO("Writing FSM symbol table '%s'\n", file);
 
     if ((fp = fopen(file, "w")) == NULL) {
-        E_ERROR("fopen(%s,r) failed\n", file);
+        E_ERROR("Failed to open symbol table '%s' for writing", file);
         return;
     }
 
