@@ -38,17 +38,15 @@
 #include <string.h>
 #include <assert.h>
 
-#include "ckd_alloc.h"
-#include "strfuncs.h"
-#include "hash_table.h"
-#include "err.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/strfuncs.h"
+#include "sphinxbase/hash_table.h"
+#include "sphinxbase/err.h"
 
 #include "jsgf_internal.h"
 #include "jsgf_parser.h"
 #include "jsgf_scanner.h"
 
-int yyparse (yyscan_t yyscanner, jsgf_t *jsgf);
-
 /**
  * \file jsgf.c
  *
@@ -231,6 +229,12 @@ extract_grammar_name(char *rule_name)
     return grammar_name;
 }
 
+char const *
+jsgf_grammar_name(jsgf_t *jsgf)
+{
+    return jsgf->name;
+}
+
 static char *
 jsgf_fullname(jsgf_t *jsgf, const char *name)
 {
@@ -503,6 +507,37 @@ jsgf_build_fsg_raw(jsgf_t *grammar, jsgf_rule_t *rule,
     return jsgf_build_fsg_internal(grammar, rule, lmath, lw, FALSE);
 }
 
+fsg_model_t *
+jsgf_read_file(const char *file, logmath_t * lmath, float32 lw)
+{
+    fsg_model_t *fsg;
+    jsgf_rule_t *rule;
+    jsgf_t *jsgf;
+    jsgf_rule_iter_t *itor;
+
+    if ((jsgf = jsgf_parse_file(file, NULL)) == NULL) {
+        E_ERROR("Error parsing file: %s\n", file);
+        return NULL;
+    }
+
+    rule = NULL;
+    for (itor = jsgf_rule_iter(jsgf); itor;
+         itor = jsgf_rule_iter_next(itor)) {
+        rule = jsgf_rule_iter_rule(itor);
+        if (jsgf_rule_public(rule)) {
+    	    jsgf_rule_iter_free(itor);
+            break;
+        }
+    }
+    if (rule == NULL) {
+        E_ERROR("No public rules found in %s\n", file);
+        return NULL;
+    }
+    fsg = jsgf_build_fsg(jsgf, rule, lmath, lw);
+    jsgf_grammar_free(jsgf);
+    return fsg;
+}
+
 int
 jsgf_write_fsg(jsgf_t *grammar, jsgf_rule_t *rule, FILE *outfh)
 {
@@ -710,8 +745,7 @@ jsgf_parse_file(const char *filename, jsgf_t *parent)
     else {
         in = fopen(filename, "r");
         if (in == NULL) {
-            fprintf(stderr, "Failed to open %s for parsing: %s\n",
-                    filename, strerror(errno));
+            E_ERROR_SYSTEM("Failed to open %s for parsing", filename);
             return NULL;
         }
         yyset_in(in, yyscanner);
@@ -720,8 +754,7 @@ jsgf_parse_file(const char *filename, jsgf_t *parent)
     jsgf = jsgf_grammar_new(parent);
     yyrv = yyparse(yyscanner, jsgf);
     if (yyrv != 0) {
-        fprintf(stderr, "JSGF parse of %s failed\n",
-                filename ? filename : "(stdin)");
+        E_ERROR("Failed to parse JSGF grammar from '%s'\n", filename ? filename : "(stdin)");
         jsgf_grammar_free(jsgf);
         yylex_destroy(yyscanner);
         return NULL;
