@@ -40,15 +40,16 @@
  *
  * Author: A cast of thousands, probably.
  */
-#include "lm3g_model.h"
-#include "listelem_alloc.h"
-#include "ckd_alloc.h"
-#include "err.h"
-
 #include <string.h>
 #include <assert.h>
 #include <limits.h>
 
+#include "sphinxbase/listelem_alloc.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/err.h"
+
+#include "lm3g_model.h"
+
 void
 lm3g_tginfo_free(ngram_model_t *base, lm3g_model_t *lm3g)
 {
@@ -160,7 +161,7 @@ lm3g_add_ug(ngram_model_t *base,
     lm3g->unigrams[wid].prob1.l = score;
     /* This unigram by definition doesn't participate in any bigrams,
      * so its backoff weight and bigram pointer are both undefined. */
-    lm3g->unigrams[wid].bo_wt1.l = logmath_get_zero(base->lmath);
+    lm3g->unigrams[wid].bo_wt1.l = 0;
     lm3g->unigrams[wid].bigrams = 0;
     /* Finally, increase the unigram count */
     ++base->n_counts[0];
@@ -174,16 +175,17 @@ lm3g_add_ug(ngram_model_t *base,
     return score;
 }
 
+#define INITIAL_SORTED_ENTRIES	MAX_UINT16
+
 void
 init_sorted_list(sorted_list_t * l)
 {
-    /* FIXME FIXME FIXME: Fixed size array!??! */
-    l->list = ckd_calloc(MAX_SORTED_ENTRIES,
-                         sizeof(sorted_entry_t));
+    l->list = ckd_calloc(INITIAL_SORTED_ENTRIES, sizeof(sorted_entry_t));
     l->list[0].val.l = INT_MIN;
     l->list[0].lower = 0;
     l->list[0].higher = 0;
     l->free = 1;
+    l->size = INITIAL_SORTED_ENTRIES;
 }
 
 void
@@ -214,11 +216,13 @@ sorted_id(sorted_list_t * l, int32 *val)
             return (i);
         if (*val < l->list[i].val.l) {
             if (l->list[i].lower == 0) {
-                if (l->free >= MAX_SORTED_ENTRIES) {
-                    /* Make the best of a bad situation. */
-                    E_WARN("sorted list overflow (%d => %d)\n",
-                           *val, l->list[i].val.l);
-                    return i;
+
+                if (l->free >= l->size) {
+            	    int newsize = l->size + INITIAL_SORTED_ENTRIES;
+            	    l->list = ckd_realloc(l->list, sizeof(sorted_entry_t) * newsize);
+            	    memset(l->list + l->size, 
+            	           0, INITIAL_SORTED_ENTRIES * sizeof(sorted_entry_t));
+            	    l->size = newsize;
                 }
 
                 l->list[i].lower = l->free;
@@ -232,11 +236,13 @@ sorted_id(sorted_list_t * l, int32 *val)
         }
         else {
             if (l->list[i].higher == 0) {
-                if (l->free >= MAX_SORTED_ENTRIES) {
-                    /* Make the best of a bad situation. */
-                    E_WARN("sorted list overflow (%d => %d)\n",
-                           *val, l->list[i].val);
-                    return i;
+
+                if (l->free >= l->size) {
+            	    int newsize = l->size + INITIAL_SORTED_ENTRIES;
+            	    l->list = ckd_realloc(l->list, sizeof(sorted_entry_t) * newsize);
+            	    memset(l->list + l->size, 
+            	           0, INITIAL_SORTED_ENTRIES * sizeof(sorted_entry_t));
+            	    l->size = newsize;
                 }
 
                 l->list[i].higher = l->free;
@@ -250,4 +256,3 @@ sorted_id(sorted_list_t * l, int32 *val)
         }
     }
 }
-
