@@ -40,23 +40,26 @@
  * Author: David Huggins-Daines, much code taken from sphinx3/src/libs3decoder/liblm
  */
 
-#include "config.h"
-#include "ngram_model.h"
-#include "ngram_model_internal.h"
-#include "ckd_alloc.h"
-#include "filename.h"
-#include "pio.h"
-#include "err.h"
-#include "logmath.h"
-#include "strfuncs.h"
-#include "case.h"
+#include <config.h>
 
 #include <string.h>
 #include <assert.h>
+
 #ifdef HAVE_ICONV
 #include <iconv.h>
 #endif 
 
+#include "sphinxbase/ngram_model.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/filename.h"
+#include "sphinxbase/pio.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/logmath.h"
+#include "sphinxbase/strfuncs.h"
+#include "sphinxbase/case.h"
+
+#include "ngram_model_internal.h"
+
 ngram_file_type_t
 ngram_file_name_to_type(const char *file_name)
 {
@@ -831,6 +834,13 @@ ngram_model_add_word(ngram_model_t *model,
 {
     int32 wid, prob = model->log_zero;
 
+    /* If we add word to unwritable model, we need to make it writable */
+    if (!model->writable) {
+	E_WARN("Can't add word '%s' to read-only language model. "
+	       "Disable mmap with '-mmap no' to make it writable\n", word);
+	return -1;
+    }
+
     wid = ngram_add_word_internal(model, word, -1);
     if (wid == NGRAM_INVALID_WID)
         return wid;
@@ -839,9 +849,7 @@ ngram_model_add_word(ngram_model_t *model,
     if (model->funcs && model->funcs->add_ug)
         prob = (*model->funcs->add_ug)(model, wid, logmath_log(model->lmath, weight));
     if (prob == 0) {
-        if (model->writable)
-            ckd_free(model->word_str[wid]);
-        return -1;
+	return -1;
     }
     return wid;
 }
