@@ -40,17 +40,18 @@
  * Author: David Huggins-Daines <dhuggins@cs.cmu.edu>
  */
 
-#include "ckd_alloc.h"
-#include "ngram_model_arpa.h"
-#include "err.h"
-#include "pio.h"
-#include "listelem_alloc.h"
-#include "strfuncs.h"
-
+#include "sphinxbase/ckd_alloc.h"
 #include <string.h>
 #include <limits.h>
 #include <assert.h>
 
+#include "sphinxbase/err.h"
+#include "sphinxbase/pio.h"
+#include "sphinxbase/listelem_alloc.h"
+#include "sphinxbase/strfuncs.h"
+
+#include "ngram_model_arpa.h"
+
 static ngram_funcs_t ngram_model_arpa_funcs;
 
 #define TSEG_BASE(m,b)		((m)->lm3g.tseg_base[(b)>>LOG_BG_SEG_SZ])
@@ -73,7 +74,7 @@ ReadNgramCounts(lineiter_t **li, int32 * n_ug, int32 * n_bg, int32 * n_tg)
         *li = lineiter_next(*li);
     }
     if (*li == NULL || strcmp((*li)->buf, "\\data\\") != 0) {
-        E_ERROR("No \\data\\ mark in LM file\n");
+        E_INFO("No \\data\\ mark in LM file\n");
         return -1;
     }
 
@@ -112,8 +113,7 @@ ReadNgramCounts(lineiter_t **li, int32 * n_ug, int32 * n_bg, int32 * n_tg)
         return -1;
     }
 
-    /* Check counts;  NOTE: #trigrams *CAN* be 0 */
-    if ((*n_ug <= 0) || (*n_bg <= 0) || (*n_tg < 0)) {
+    if ((*n_ug <= 0) || (*n_bg < 0) || (*n_tg < 0)) {
         E_ERROR("Bad or missing ngram count\n");
         return -1;
     }
@@ -141,7 +141,8 @@ ReadUnigrams(lineiter_t **li, ngram_model_arpa_t * model)
         int n;
 
         string_trim((*li)->buf, STRING_BOTH);
-        if (strcmp((*li)->buf, "\\2-grams:") == 0)
+        if (strcmp((*li)->buf, "\\2-grams:") == 0
+            || strcmp((*li)->buf, "\\end\\") == 0)
             break;
 
         if ((n = str2words((*li)->buf, wptr, 3)) < 2) {
@@ -373,7 +374,7 @@ ReadTrigrams(lineiter_t **li, ngram_model_arpa_t * model)
                 if (prev_seg >= 0) {
                     tgoff = tgcount - model->lm3g.tseg_base[prev_seg];
                     if (tgoff > 65535) {
-                        E_ERROR("Offset from tseg_base > 65535\n");
+                        E_ERROR("Size of trigram segment is bigger than 65535, such a big language models are not supported, use smaller vocabulary\n");
                         return -1;
                     }
                 }
@@ -392,7 +393,7 @@ ReadTrigrams(lineiter_t **li, ngram_model_arpa_t * model)
 
                 tgoff = tgcount - model->lm3g.tseg_base[prev_seg];
                 if (tgoff > 65535) {
-                    E_ERROR("Offset from tseg_base > 65535\n");
+                    E_ERROR("Size of trigram segment is bigger than 65535, such a big language models are not supported, use smaller vocabulary\n");
                     return -1;
                 }
 
@@ -423,7 +424,7 @@ ReadTrigrams(lineiter_t **li, ngram_model_arpa_t * model)
         if ((prev_bg & (BG_SEG_SZ - 1)) == 0)
             model->lm3g.tseg_base[prev_bg >> LOG_BG_SEG_SZ] = tgcount;
         if ((tgcount - model->lm3g.tseg_base[prev_bg >> LOG_BG_SEG_SZ]) > 65535) {
-            E_ERROR("Offset from tseg_base > 65535\n");
+            E_ERROR("Size of trigram segment is bigger than 65535, such a big language models are not supported, use smaller vocabulary\n");
             return -1;
         }
         model->lm3g.bigrams[prev_bg].trigrams =
@@ -518,18 +519,20 @@ ngram_model_arpa_read(cmd_ln_t *config,
     if (base->n_counts[2] > 0)
         init_sorted_list(&model->sorted_bo_wt2);
 
-    if (ReadBigrams(&li, model) == -1) {
-        fclose_comp(fp, is_pipe);
-        ngram_model_free(base);
-        return NULL;
-    }
+    if (base->n_counts[1] > 0) {
+        if (ReadBigrams(&li, model) == -1) {
+            fclose_comp(fp, is_pipe);
+            ngram_model_free(base);
+            return NULL;
+        }
 
-    base->n_counts[1] = FIRST_BG(model, base->n_counts[0]);
-    model->lm3g.n_prob2 = model->sorted_prob2.free;
-    model->lm3g.prob2 = vals_in_sorted_list(&model->sorted_prob2);
-    free_sorted_list(&model->sorted_prob2);
-    E_INFO("%8d = #bigrams created\n", base->n_counts[1]);
-    E_INFO("%8d = #prob2 entries\n", model->lm3g.n_prob2);
+        base->n_counts[1] = FIRST_BG(model, base->n_counts[0]);
+        model->lm3g.n_prob2 = model->sorted_prob2.free;
+        model->lm3g.prob2 = vals_in_sorted_list(&model->sorted_prob2);
+        free_sorted_list(&model->sorted_prob2);
+        E_INFO("%8d = #bigrams created\n", base->n_counts[1]);
+        E_INFO("%8d = #prob2 entries\n", model->lm3g.n_prob2);
+    }
 
     if (base->n_counts[2] > 0) {
         /* Create trigram bo-wts array */
