@@ -43,9 +43,10 @@
 #ifndef __NGRAM_MODEL_DMP_H__
 #define __NGRAM_MODEL_DMP_H__
 
+#include "sphinxbase/mmio.h"
+
 #include "ngram_model_internal.h"
 #include "lm3g_model.h"
-#include "mmio.h"
 
 /**
  * On-disk representation of bigrams.
