@@ -40,7 +40,8 @@
  * Author: David Huggins-Daines <dhuggins@cs.cmu.edu>
  */
 
-#include "ckd_alloc.h"
+#include "sphinxbase/ckd_alloc.h"
+
 #include "ngram_model_internal.h"
 
 static ngram_funcs_t ngram_model_dmp32_funcs;
