@@ -75,12 +75,12 @@
 #include <unistd.h>
 #endif
 
-#include "cmd_ln.h"
-#include "err.h"
-#include "ckd_alloc.h"
-#include "hash_table.h"
-#include "case.h"
-#include "strfuncs.h"
+#include "sphinxbase/cmd_ln.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/ckd_alloc.h"
+#include "sphinxbase/hash_table.h"
+#include "sphinxbase/case.h"
+#include "sphinxbase/strfuncs.h"
 
 typedef struct cmd_ln_val_s {
     anytype_t val;
@@ -240,6 +240,7 @@ arg_dump_r(cmd_ln_t *cmdln, FILE * fp, const arg_t * defn, int32 doc)
     int32 i, l, n;
     int32 namelen, deflen;
     anytype_t *vp;
+    char const **array;
 
     /* No definitions, do nothing. */
     if (defn == NULL)
@@ -306,6 +307,13 @@ arg_dump_r(cmd_ln_t *cmdln, FILE * fp, const arg_t * defn, int32 doc)
                     if (vp->ptr)
                         fprintf(fp, "%s", (char *)vp->ptr);
                     break;
+                case ARG_STRING_LIST:
+            	    array = (char const**)vp->ptr;
+            	    if (array)
+            		for (l = 0; array[l] != 0; l++) {
+            		    fprintf(fp, "%s,", array[l]);
+            		}
+            	    break;
                 case ARG_BOOLEAN:
                 case REQARG_BOOLEAN:
                     fprintf(fp, "%s", vp->i ? "yes" : "no");
@@ -324,6 +332,32 @@ arg_dump_r(cmd_ln_t *cmdln, FILE * fp, const arg_t * defn, int32 doc)
     fflush(fp);
 }
 
+static char **
+parse_string_list(const char *str)
+{
+    int count, i, j;
+    const char *p;
+    char ** result;
+
+    p = str;
+    count = 1;
+    while (*p) {
+	if (*p == ',')
+	    count++;
+	p++;
+    }
+    /* Should end with NULL */
+    result = (char **) ckd_calloc(count + 1, sizeof(char *));
+    p = str;
+    for (i = 0; i < count; i++) {
+	for (j = 0; p[j] != ',' && p[j] != 0; j++);
+	result[i] = ckd_calloc(j + 1, sizeof(char));
+	strncpy( result[i], p, j);
+	p = p + j + 1;
+    }
+    return result;
+}
+
 static cmd_ln_val_t *
 cmd_ln_val_init(int t, const char *str)
 {
@@ -371,6 +405,9 @@ cmd_ln_val_init(int t, const char *str)
         case REQARG_STRING:
             val.ptr = ckd_salloc(e_str);
             break;
+        case ARG_STRING_LIST:
+    	    val.ptr = parse_string_list(e_str);
+            break;
         default:
             E_ERROR("Unknown argument type: %d\n", t);
             valid = 0;
@@ -432,6 +469,16 @@ parse_options(cmd_ln_t *cmdln, const arg_t *defn, int32 argc, char* argv[], int3
 void
 cmd_ln_val_free(cmd_ln_val_t *val)
 {
+    int i;
+    if (val->type & ARG_STRING_LIST) {
+	char ** array = (char **)val->val.ptr;
+	if (array) {
+	    for (i = 0; array[i] != NULL; i++) {
+		ckd_free(array[i]);
+	    }
+	    ckd_free(array);
+	}
+    }
     if (val->type & ARG_STRING)
         ckd_free(val->val.ptr);
     ckd_free(val);
@@ -503,22 +550,10 @@ cmd_ln_appl_exit()
 cmd_ln_t *
 cmd_ln_parse_r(cmd_ln_t *inout_cmdln, const arg_t * defn, int32 argc, char *argv[], int strict)
 {
-    int32 i, j, n;
+    int32 i, j, n, argstart;
     hash_table_t *defidx = NULL;
     cmd_ln_t *cmdln;
-
-    /* Echo command line */
-#ifndef _WIN32_WCE
-    E_INFO("Parsing command line:\n");
-    for (i = 0; i < argc; i++) {
-        if (argv[i][0] == '-')
-            E_INFOCONT("\\\n\t");
-        E_INFOCONT("%s ", argv[i]);
-    }
-    E_INFOCONT("\n\n");
-    fflush(stderr);
-#endif
-
+    
     /* Construct command-line object */
     if (inout_cmdln == NULL) {
         cmdln = ckd_calloc(1, sizeof(*cmdln));
@@ -549,17 +584,19 @@ cmd_ln_parse_r(cmd_ln_t *inout_cmdln, const arg_t * defn, int32 argc, char *argv
     if (cmdln->ht == NULL)
         cmdln->ht = hash_table_new(n, 0 /* argument names are case-sensitive */ );
 
-    /* Parse command line arguments (name-value pairs); skip argv[0] if argc is odd */
-    for (j = argc % 2; j < argc; j += 2) {
+
+    /* skip argv[0] if it doesn't start with dash */
+    argstart = 0;
+    if (argc > 0 && argv[0][0] != '-') {
+        argstart = 1;
+    } 
+
+    /* Parse command line arguments (name-value pairs) */
+    for (j = argstart; j < argc; j += 2) {
         arg_t *argdef;
         cmd_ln_val_t *val;
         void *v;
 
-        if (j + 1 >= argc) {
-            cmd_ln_print_help_r(cmdln, stderr, defn);
-            E_ERROR("Argument value for '%s' missing\n", argv[j]);
-            goto error;
-        }
         if (hash_table_lookup(defidx, argv[j], &v) < 0) {
             if (strict) {
                 E_ERROR("Unknown argument name '%s'\n", argv[j]);
@@ -572,7 +609,13 @@ cmd_ln_parse_r(cmd_ln_t *inout_cmdln, const arg_t * defn, int32 argc, char *argv
         }
         argdef = v;
 
-        /* Enter argument value */
+        /* Enter argument value */	
+	if (j + 1 >= argc) {
+            cmd_ln_print_help_r(cmdln, stderr, defn);
+            E_ERROR("Argument value for '%s' missing\n", argv[j]);
+            goto error;
+        }
+
         if (argdef == NULL)
             val = cmd_ln_val_init(ARG_STRING, argv[j + 1]);
         else {
@@ -629,16 +672,36 @@ cmd_ln_parse_r(cmd_ln_t *inout_cmdln, const arg_t * defn, int32 argc, char *argv
     }
 
     if (strict && argc == 1) {
-        E_ERROR("No arguments given, exiting\n");
+        E_ERROR("No arguments given, available options are:\n");
         cmd_ln_print_help_r(cmdln, stderr, defn);
-        goto error;
+        if (defidx)
+	    hash_table_free(defidx);
+        if (inout_cmdln == NULL)
+	    cmd_ln_free_r(cmdln);
+        return NULL;
     }
 
 #ifndef _WIN32_WCE
+    /* Set up logging. We need to do this earlier because we want to dump
+     * the information to the configured log, not to the stderr. */
+    if (cmd_ln_exists_r(cmdln, "-logfn") && cmd_ln_str_r(cmdln, "-logfn"))
+        err_set_logfile(cmd_ln_str_r(cmdln, "-logfn"));
+
+    /* Echo command line */
+    E_INFO("Parsing command line:\n");
+    for (i = 0; i < argc; i++) {
+        if (argv[i][0] == '-')
+            E_INFOCONT("\\\n\t");
+        E_INFOCONT("%s ", argv[i]);
+    }
+    E_INFOCONT("\n\n");
+    fflush(stderr);
+
     /* Print configuration */
     E_INFOCONT("Current configuration:\n");
     arg_dump_r(cmdln, err_get_logfp(), defn, 0);
 #endif
+
     hash_table_free(defidx);
     return cmdln;
 
@@ -647,7 +710,7 @@ cmd_ln_parse_r(cmd_ln_t *inout_cmdln, const arg_t * defn, int32 argc, char *argv
         hash_table_free(defidx);
     if (inout_cmdln == NULL)
         cmd_ln_free_r(cmdln);
-    E_ERROR("cmd_ln_parse_r failed\n");
+    E_ERROR("Failed to parse arguments list\n");
     return NULL;
 }
 
@@ -696,7 +759,7 @@ cmd_ln_parse(const arg_t * defn, int32 argc, char *argv[], int strict)
     cmdln = cmd_ln_parse_r(global_cmdln, defn, argc, argv, strict);
     if (cmdln == NULL) {
         /* Old, bogus behaviour... */
-        E_ERROR("cmd_ln_parse failed, forced exit\n");
+        E_ERROR("Failed to parse arguments list, forced exit\n");
         exit(-1);
     }
     /* Initialize global_cmdln if not present. */
@@ -892,6 +955,16 @@ cmd_ln_str_r(cmd_ln_t *cmdln, char const *name)
     return (char const *)val->ptr;
 }
 
+char const **
+cmd_ln_str_list_r(cmd_ln_t *cmdln, char const *name)
+{
+    anytype_t *val;
+    val = cmd_ln_access_r(cmdln, name);
+    if (val == NULL)
+        return NULL;
+    return (char const **)val->ptr;
+}
+
 long
 cmd_ln_int_r(cmd_ln_t *cmdln, char const *name)
 {
