@@ -80,11 +80,11 @@
 #define NO_LONG_LONG
 #endif
 #define Omit_Private_Memory
-#include "ckd_alloc.h"
+#include "sphinxbase/ckd_alloc.h"
 #undef USE_LOCALE
 
 /* Correct totally bogus typedefs in this code. */
-#include "prim_type.h"
+#include "sphinxbase/prim_type.h"
 #define Long int32   /* ZOMG */
 #define ULong uint32 /* WTF */
 
@@ -2391,3 +2391,7 @@ sb_strtod
 		*se = (char *)s;
 	return sign ? -dval(rv) : dval(rv);
 	}
+
+#ifdef __cplusplus
+}
+#endif
