@@ -39,13 +39,15 @@
  * @brief Somewhat antiquated logging and error interface.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <errno.h>
 
-#include "err.h"
-#include "config.h"
+#include "sphinxbase/err.h"
 
 #ifdef SPHINX_DEBUG
 static int sphinx_debug_level;
@@ -84,7 +86,6 @@ void
 logfp_index_alloc(void)
 {
     pthread_key_create(&logfp_index, NULL);
-    pthread_setspecific(logfp_index, (void *)-1);
 }
 
 FILE *
@@ -94,8 +95,11 @@ err_get_logfp(void)
 
     pthread_once(&logfp_index_once, logfp_index_alloc);
     logfp = (FILE *)pthread_getspecific(logfp_index);
-    if (logfp == (FILE *)-1)
+
+    if (logfp == NULL)
         return stderr;
+    else if (logfp == (FILE*) -1)
+	return NULL;
     else
         return logfp;
 }
@@ -103,6 +107,9 @@ err_get_logfp(void)
 static void
 internal_set_logfp(FILE *fh)
 {
+    if (fh == NULL)
+	fh = (FILE*) -1;
+
     pthread_setspecific(logfp_index, (void *)fh);
 }
 
@@ -115,7 +122,6 @@ void
 logfp_index_alloc(void)
 {
     logfp_index = TlsAlloc();
-    TlsSetValue(logfp_index, (void *)-1);
 }
 
 FILE *
@@ -126,8 +132,11 @@ err_get_logfp(void)
     if (InterlockedExchange(&logfp_index_once, 1) == 0)
         logfp_index_alloc();
     logfp = (FILE *)TlsGetValue(logfp_index);
-    if (logfp == (FILE *)-1)
+
+    if (logfp == NULL)
         return stderr;
+    else if (logfp == (FILE*) -1)
+	return NULL;
     else
         return logfp;
 }
@@ -135,17 +144,22 @@ err_get_logfp(void)
 static void
 internal_set_logfp(FILE *fh)
 {
+    if (fh == NULL)
+	fh = (FILE*) -1;
+
     TlsSetValue(logfp_index, (void *)fh);
 }
 
 #else
-FILE *logfp = (FILE *)-1;
+FILE *logfp = NULL;
 
 FILE *
 err_get_logfp(void)
 {
-    if (logfp == (FILE *)-1)
+    if (logfp == NULL)
         return stderr;
+    else if (logfp == (FILE*) -1)
+	return NULL;
     else
         return logfp;
 }
@@ -153,6 +167,9 @@ err_get_logfp(void)
 static void
 internal_set_logfp(FILE *fh)
 {
+    if (fh == NULL)
+	fh = (FILE*) -1;
+
     logfp = fh;
 }
 
@@ -296,7 +313,7 @@ _E__fatal_sys_error(char const *fmt, ...)
         vfprintf(logfp, fmt, pvar);
         va_end(pvar);
 
-        fprintf(logfp, "; %s\n", strerror(local_errno));
+        fprintf(logfp, ": %s\n", strerror(local_errno));
         fflush(logfp);
     }
 
@@ -324,7 +341,7 @@ _E__sys_error(char const *fmt, ...)
     vfprintf(logfp, fmt, pvar);
     va_end(pvar);
 
-    fprintf(logfp, "; %s\n", strerror(local_errno));
+    fprintf(logfp, ": %s\n", strerror(local_errno));
     fflush(logfp);
 }
 
