@@ -44,31 +44,33 @@
  * 
  *********************************************************************/
 
-#include "prim_type.h"
-#include "err.h"
-#include "mmio.h"
-#include "ckd_alloc.h"
-
 #include <string.h>
 #include <stdlib.h>
 
-#if (defined(_WIN32) || defined(GNUWINCE)) && !defined(__SYMBIAN32__)
-# ifdef GNUWINCE
-#  include <sys/wcebase.h>
-#  include <sys/wcetypes.h>
-#  include <sys/wcememory.h>
-#  include <sys/wcefile.h>
-# else /* !GNUWINCE */
-#  include <windows.h>
-# endif /* !GNUWINCE */
-#elif defined(__ADSPBLACKFIN__) && !defined(__linux__)
-#else /* !_WIN32 */
+#ifdef GNUWINCE
+# include <sys/wcebase.h>
+# include <sys/wcetypes.h>
+# include <sys/wcememory.h>
+# include <sys/wcefile.h>
+#elif defined(__SYMBIAN32__) /* SYMBIAN32 must be before WIN32 since Symbian SDK defines WIN32 as well */
+# include <unistd.h>
+# include <fcntl.h>
+# include <sys/stat.h>
+# include <sys/mman.h>
+#elif defined(_WIN32)
+# include <windows.h>
+#else
 # include <unistd.h>
 # include <fcntl.h>
 # include <sys/stat.h>
 # include <sys/file.h>
 # include <sys/mman.h>
-#endif /* !_WIN32 */
+#endif
+
+#include "sphinxbase/prim_type.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/mmio.h"
+#include "sphinxbase/ckd_alloc.h"
 
 /** Silvio Moioli: slightly updated */
 #if defined(_WIN32_WCE) || defined(GNUWINCE)
@@ -92,7 +94,7 @@ mmio_file_read(const char *filename)
          CreateFileForMappingW(wfilename, GENERIC_READ, FILE_SHARE_READ,
                                NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
                                NULL)) == INVALID_HANDLE_VALUE) {
-        E_ERROR("Failed to CreateFileForMapping(%s): %08x\n", filename,
+        E_ERROR("Failed to create mapping for the file '%s': %08x\n", filename,
                 GetLastError());
         return NULL;
     }
@@ -138,7 +140,7 @@ mmio_file_read(const char *filename)
     if ((ffm = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ,
                          NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
                          NULL)) == INVALID_HANDLE_VALUE) {
-        E_ERROR("Failed to CreateFile(%s): %08x\n",
+        E_ERROR("Failed to create file '%s': %08x\n",
                 filename, GetLastError());
         return NULL;
     }
@@ -215,11 +217,13 @@ mmio_file_read(const char *filename)
     }
     if (fstat(fd, &buf) == -1) {
         E_ERROR_SYSTEM("Failed to stat %s", filename);
+        close(fd);
         return NULL;
     }
     ptr = mmap(NULL, buf.st_size, PROT_READ, MAP_SHARED, fd, 0);
     if (ptr == (void *)-1) {
-        E_ERROR("Failed to mmap %d bytes", buf.st_size);
+        E_ERROR_SYSTEM("Failed to mmap %lld bytes", (unsigned long long)buf.st_size);
+        close(fd);
         return NULL;
     }
     close(fd);
