@@ -51,11 +51,11 @@
 #endif
 #include <assert.h>
 
-#include "pio.h"
-#include "filename.h"
-#include "err.h"
-#include "strfuncs.h"
-#include "ckd_alloc.h"
+#include "sphinxbase/pio.h"
+#include "sphinxbase/filename.h"
+#include "sphinxbase/err.h"
+#include "sphinxbase/strfuncs.h"
+#include "sphinxbase/ckd_alloc.h"
 
 #ifndef EXEEXT
 #define EXEEXT ""
@@ -128,7 +128,7 @@ fopen_comp(const char *file, const char *mode, int32 * ispipe)
                 E_FATAL("Unknown  compression type %d\n", isgz);
             }
             if ((fp = popen(command, mode)) == NULL) {
-                E_ERROR_SYSTEM("popen (%s,%s) failed\n", command, mode);
+                E_ERROR_SYSTEM("Failed to open a pipe for a command '%s' mode '%s'", command, mode);
                 ckd_free(command);
                 return NULL;
             }
@@ -151,14 +151,14 @@ fopen_comp(const char *file, const char *mode, int32 * ispipe)
                 E_FATAL("Unknown compression type %d\n", isgz);
             }
             if ((fp = popen(command, mode)) == NULL) {
-                E_ERROR_SYSTEM("popen (%s,%s) failed\n", command, mode);
+                E_ERROR_SYSTEM("Failed to open a pipe for a command '%s' mode '%s'", command, mode);
                 ckd_free(command);
                 return NULL;
             }
             ckd_free(command);
         }
         else {
-            E_ERROR("fopen_comp not implemented for mode = %s\n", mode);
+            E_ERROR("Compressed file operation for mode %s is not supported", mode);
             return NULL;
         }
 #endif /* HAVE_POPEN */
@@ -176,7 +176,7 @@ fclose_comp(FILE * fp, int32 ispipe)
 {
     if (ispipe) {
 #ifdef HAVE_POPEN
-#if defined(_WIN32)
+#if defined(_WIN32) && (!defined(__SYMBIAN32__))
         _pclose(fp);
 #else
         pclose(fp);
@@ -263,12 +263,46 @@ lineiter_start(FILE *fh)
     li->len = 0;
     li->fh = fh;
 
-    return lineiter_next(li);
+    li = lineiter_next(li);
+    
+    /* Strip the UTF-8 BOM */
+    
+    if (li && 0 == strncmp(li->buf, "\xef\xbb\xbf", 3)) {
+	memmove(li->buf, li->buf + 3, strlen(li->buf + 1));
+	li->len -= 3;
+    }
+    
+    return li;
 }
 
 lineiter_t *
-lineiter_next(lineiter_t *li)
+lineiter_start_clean(FILE *fh)
 {
+    lineiter_t *li;
+    
+    li = lineiter_start(fh);
+    
+    if (li == NULL)
+	return li;
+    
+    li->clean = TRUE;
+    
+    if (li->buf && li->buf[0] == '#') {
+	li = lineiter_next(li);
+    } else {
+	string_trim(li->buf, STRING_BOTH);
+    }
+    
+    return li;
+}
+
+
+static lineiter_t *
+lineiter_next_plain(lineiter_t *li)
+{
+    /* We are reading the next line */
+    li->lineno++;
+    
     /* Read a line and check for EOF. */
     if (fgets(li->buf, li->bsiz, li->fh) == NULL) {
         lineiter_free(li);
@@ -299,6 +333,27 @@ lineiter_next(lineiter_t *li)
     return li;
 }
 
+
+lineiter_t *
+lineiter_next(lineiter_t *li)
+{
+    if (!li->clean)
+	return lineiter_next_plain(li);
+    
+    for (li = lineiter_next_plain(li); li; li = lineiter_next_plain(li)) {
+	if (li->buf && li->buf[0] != '#') {
+	    li->buf = string_trim(li->buf, STRING_BOTH);
+	    break;
+	}
+    }
+    return li;
+}
+
+int lineiter_lineno(lineiter_t *li)
+{
+    return li->lineno;
+}
+
 void
 lineiter_free(lineiter_t *li)
 {
@@ -337,7 +392,6 @@ fread_line(FILE *stream, size_t *out_len)
     return output;
 }
 
-
 #define FREAD_RETRY_COUNT	60
 
 int32
@@ -447,7 +501,7 @@ stat_retry(const char *file, struct stat * statbuf)
             return 0;
 #endif
         if (i == 0) {
-            E_ERROR_SYSTEM("stat(%s) failed; retrying...\n", file);
+            E_ERROR_SYSTEM("Failed to stat file '%s'; retrying...", file);
         }
 #ifdef HAVE_UNISTD_H
         sleep(1);
@@ -559,7 +613,7 @@ bit_encode_flush(bit_encode_t *be)
     return 0;
 }
 
-#ifdef HAVE_SYS_STAT_H /* Unix, Cygwin */
+#if defined(HAVE_SYS_STAT_H) && !defined(__MINGW32__) /* Unix, Cygwin, doesn't work on MINGW */
 int
 build_directory(const char *path)
 {
