@@ -32,47 +32,11 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * ====================================================================
- *
- * 
- * HISTORY
- * 
- * circa 1994	P J Moreno at Carnegie Mellon
- * 		Created.
- *
- * For history information, please use 'cvs log'
- * $Log$
- * Revision 1.11  2006/02/24  04:06:43  arthchan2003
- * Merged from branch SPHINX3_5_2_RCI_IRII_BRANCH: Changed commands to macro.  Used E_INFO instead of printf in displaying no. of friends
- * 
- *
- * Revision 1.10  2005/08/18 21:18:09  egouvea
- * Added E_INFO displaying information about how many columns are being printed, and how many frames
- *
- * Revision 1.9.4.2  2005/09/07 23:51:05  arthchan2003
- * Fixed keyword expansion problem
- *
- * Revision 1.9.4.1  2005/07/18 23:21:23  arthchan2003
- * Tied command-line arguments with marcos
- *
- * Revision 1.10  2005/08/18 21:18:09  egouvea
- * Added E_INFO displaying information about how many columns are being printed, and how many frames
- *
- * Revision 1.9  2005/06/22 05:38:45  arthchan2003
- * Add
- *
- * Revision 1.2  2005/03/30 00:43:41  archan
- *
- * Add $Log$
- * Revision 1.11  2006/02/24  04:06:43  arthchan2003
- * Merged from branch SPHINX3_5_2_RCI_IRII_BRANCH: Changed commands to macro.  Used E_INFO instead of printf in displaying no. of friends
- * 
- *
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strfuncs.h>
 
 #ifdef _WIN32
 #pragma warning (disable: 4996) 
@@ -82,15 +46,14 @@
 #include <config.h>
 #endif
 
-#include "prim_type.h"
-#include "cmd_ln.h"
-#include "ckd_alloc.h"
-#include "info.h"
-#include "err.h"
-#include "bio.h"
-
-/* Silvio Moioli: switched to stat_reply that's Windows CE friendly. */
-#include "pio.h"
+#include <sphinxbase/strfuncs.h>
+#include <sphinxbase/prim_type.h>
+#include <sphinxbase/cmd_ln.h>
+#include <sphinxbase/ckd_alloc.h>
+#include <sphinxbase/info.h>
+#include <sphinxbase/err.h>
+#include <sphinxbase/bio.h>
+#include <sphinxbase/pio.h>
 
 /** \file main_cepview.c
     \brief Main driver of cepview
@@ -185,7 +148,7 @@ main(int argc, char *argv[])
         E_FATAL("Input file was not specified with (-f)\n");
     }
     if (read_cep(cepfile, &cep, &noframe, vsize) == IO_ERR)
-        E_FATAL("ERROR opening %s for reading\n", cepfile);
+        E_FATAL_SYSTEM("Failed to open '%s' for reading", cepfile);
 
     z = cep[0];
 
@@ -235,16 +198,16 @@ read_cep(char const *file, float ***cep, int *numframes, int cepsize)
     FILE *fp;
     int n_float;
     struct stat statbuf;
-    int i, n, byterev, sf, ef;
+    int i, n, byterev;
     float32 **mfcbuf;
 
     if (stat_retry(file, &statbuf) < 0) {
-        printf("stat(%s) failed\n", file);
+        E_ERROR_SYSTEM("Failed to get file size '%s'", file);
         return IO_ERR;
     }
 
     if ((fp = fopen(file, "rb")) == NULL) {
-        printf("fopen(%s, rb) failed\n", file);
+        E_ERROR_SYSTEM("Failed to open '%s' for reading", file);
         return IO_ERR;
     }
 
@@ -261,8 +224,7 @@ read_cep(char const *file, float ***cep, int *numframes, int cepsize)
         SWAP_INT32(&n);
 
         if ((int) (n * sizeof(float) + 4) != statbuf.st_size) {
-            printf
-                ("Header size field: %d(%08x); filesize: %d(%08x)\n",
+    	    E_ERROR("Header size field: %d(%08x); filesize: %d(%08x)\n",
                  n_float, n_float, (int) statbuf.st_size,
                  (int) statbuf.st_size);
             fclose(fp);
@@ -273,7 +235,7 @@ read_cep(char const *file, float ***cep, int *numframes, int cepsize)
         byterev = TRUE;
     }
     if (n_float <= 0) {
-        printf("Header size field: %d\n", n_float);
+        E_ERROR("Header size field: %d\n", n_float);
         fclose(fp);
         return IO_ERR;
     }
@@ -281,20 +243,18 @@ read_cep(char const *file, float ***cep, int *numframes, int cepsize)
     /* n = #frames of input */
     n = n_float / cepsize;
     if (n * cepsize != n_float) {
-        printf("Header size field: %d; not multiple of %d\n",
+        E_ERROR("Header size field: %d; not multiple of %d\n",
                n_float, cepsize);
         fclose(fp);
         return IO_ERR;
     }
-    sf = 0;
-    ef = n;
 
     mfcbuf = (float **) ckd_calloc_2d(n, cepsize, sizeof(float32));
 
     /* Read mfc data and byteswap if necessary */
     n_float = n * cepsize;
     if ((int) fread(mfcbuf[0], sizeof(float), n_float, fp) != n_float) {
-        printf("Error reading mfc data\n");
+        E_ERROR("Error reading mfc data from the file '%s'", file);
         fclose(fp);
         return IO_ERR;
     }
