@@ -47,8 +47,8 @@
 #ifndef CMD_LN_DEFN_H
 #define CMD_LN_DEFN_H
 
-#include "cmd_ln.h"
-#include "fe.h"
+#include <sphinxbase/cmd_ln.h>
+#include <sphinxbase/fe.h>
 
 const char helpstr[] =
   "Description: \n\
@@ -155,6 +155,11 @@ static arg_t defn[] = {
     "yes",
     "Create missing subdirectories in output directory" },
 
+  { "-sph2pipe",
+    ARG_BOOLEAN,
+    "no",
+    "Input is NIST sphere (possibly with Shorten), use sph2pipe to convert" },
+
   { "-nist",
     ARG_BOOLEAN,
     "no",
@@ -170,6 +175,13 @@ static arg_t defn[] = {
     "no",
     "Defines input format as Microsoft Wav (RIFF)" },
   
+#ifdef HAVE_SNDFILE_H
+  { "-sndfile",
+    ARG_BOOLEAN,
+    "no",
+    "Use libsndfile to read input data" },
+#endif
+  
   { "-nchans",
     ARG_INT32,
     "1",
@@ -177,8 +189,8 @@ static arg_t defn[] = {
   
   { "-whichchan",
     ARG_INT32,
-    "1",
-    "Channel to process" },
+    "0",
+    "Channel to process (numbered from 1), or 0 to mix all channels" },
   
   { "-ofmt",
     ARG_STRING,
