@@ -44,21 +44,26 @@
 #include <config.h>
 #endif
 
-#include "fe.h"
-#include "strfuncs.h"
-#include "pio.h"
-#include "filename.h"
-#include "cmd_ln.h"
-#include "err.h"
-#include "ckd_alloc.h"
-#include "byteorder.h"
+#ifdef HAVE_SNDFILE_H
+#include <sndfile.h>
+#endif
+
+#include <sphinxbase/fe.h>
+#include <sphinxbase/strfuncs.h>
+#include <sphinxbase/pio.h>
+#include <sphinxbase/filename.h>
+#include <sphinxbase/cmd_ln.h>
+#include <sphinxbase/err.h>
+#include <sphinxbase/ckd_alloc.h>
+#include <sphinxbase/byteorder.h>
+#include <sphinxbase/hash_table.h>
 
 #include "sphinx_wave2feat.h"
 #include "cmd_ln_defn.h"
 
 typedef struct audio_type_s {
     char const *name;
-    int (*detect)(sphinx_wave2feat_t *wtf, char const *infile);
+    int (*detect)(sphinx_wave2feat_t *wtf);
     int (*decode)(sphinx_wave2feat_t *wtf);
 } audio_type_t;
 
@@ -83,6 +88,9 @@ struct sphinx_wave2feat_s {
     int veclen;       /**< Length of each output vector. */
     int in_veclen;    /**< Length of each input vector (for cep<->spec). */
     int byteswap;     /**< Whether byteswapping is necessary. */
+#ifdef HAVE_SNDFILE_H
+    SNDFILE *insfh;   /**< Input sndfile handle. */
+#endif
     output_type_t const *ot;/**< Output type object. */
 };
 
@@ -108,13 +116,13 @@ typedef struct RIFFHeader{
  * @return TRUE if it's a RIFF file, FALSE if not, -1 if an error occurred.
  */
 static int
-detect_riff(sphinx_wave2feat_t *wtf, char const *infile)
+detect_riff(sphinx_wave2feat_t *wtf)
 {
     FILE *fh;
     MSWAV_hdr hdr;
 
-    if ((fh = fopen(infile, "rb")) == NULL) {
-        E_ERROR_SYSTEM("Failed to open %s", infile);
+    if ((fh = fopen(wtf->infile, "rb")) == NULL) {
+        E_ERROR_SYSTEM("Failed to open %s", wtf->infile);
         return -1;
     }
     if (fread(&hdr, sizeof(hdr), 1, fh) != 1) {
@@ -123,25 +131,21 @@ detect_riff(sphinx_wave2feat_t *wtf, char const *infile)
         return -1;
     }
     /* Make sure it is actually a RIFF file. */
-    if (0 != memcmp(hdr.rifftag, "RIFF", 4))
+    if (0 != memcmp(hdr.rifftag, "RIFF", 4)) {
+        fclose(fh);
         return FALSE;
+    }
 
     /* Get relevant information. */
     cmd_ln_set_int32_r(wtf->config, "-nchans", hdr.numchannels);
     cmd_ln_set_float32_r(wtf->config, "-samprate", hdr.SamplingFreq);
-    wtf->infile = ckd_salloc(infile);
     wtf->infh = fh;
 
     return TRUE;
 }
 
-/**
- * Detect NIST file and parse its header if detected.
- *
- * @return TRUE if it's a NIST file, FALSE if not, -1 if an error occurred.
- */
 static int
-detect_nist(sphinx_wave2feat_t *wtf, char const *infile)
+open_nist_file(sphinx_wave2feat_t *wtf, char const *infile, FILE **out_fh, int detect_endian)
 {
     char nist[7];
     lineiter_t *li;
@@ -168,8 +172,10 @@ detect_nist(sphinx_wave2feat_t *wtf, char const *infile)
         int nword;
 
         string_trim(li->buf, STRING_BOTH);
-        if (strlen(li->buf) == 0)
+        if (strlen(li->buf) == 0) {
+            lineiter_free(li);
             break;
+        }
         nword = str2words(li->buf, NULL, 0);
         if (nword != 3)
             continue;
@@ -179,20 +185,72 @@ detect_nist(sphinx_wave2feat_t *wtf, char const *infile)
             cmd_ln_set_float32_r(wtf->config, "-samprate", atof_c(words[2]));
         }
         if (0 == strcmp(words[0], "channel_count")) {
-            cmd_ln_set_float32_r(wtf->config, "-nchans", atoi(words[2]));
+            cmd_ln_set_int32_r(wtf->config, "-nchans", atoi(words[2]));
         }
-        if (0 == strcmp(words[0], "sample_byte_format")) {
+        if (detect_endian && 0 == strcmp(words[0], "sample_byte_format")) {
             cmd_ln_set_str_r(wtf->config, "-input_endian",
                              (0 == strcmp(words[2], "10")) ? "big" : "little");
         }
-        /* FIMXE: Warn about shorten-compressed data. */
         ckd_free(words);
     }
+
     fseek(fh, 1024, SEEK_SET);
-    wtf->infile = ckd_salloc(infile);
+    if (out_fh)
+        *out_fh = fh;
+    else
+        fclose(fh);
+    return TRUE;
+}
+
+#ifdef HAVE_POPEN
+static int
+detect_sph2pipe(sphinx_wave2feat_t *wtf)
+{
+    FILE *fh;
+    char *cmdline;
+    int rv;
+
+    /* Determine if it's NIST file and get parameters. */
+    if ((rv = open_nist_file(wtf, wtf->infile, NULL, FALSE)) != TRUE)
+        return rv;
+
+    /* Now popen it with sph2pipe. */
+    cmdline = string_join("sph2pipe -f raw '", wtf->infile, "'", NULL);
+    if ((fh = popen(cmdline, "r")) == NULL) {
+        E_ERROR_SYSTEM("Failed to popen(\"sph2pipe -f raw '%s'\")", wtf->infile);
+        ckd_free(cmdline);
+        return -1;
+    }
+
     wtf->infh = fh;
     return TRUE;
 }
+#else /* !HAVE_POPEN */
+static int
+detect_sph2pipe(sphinx_wave2feat_t *wtf)
+{
+    E_ERROR("popen() not available, cannot run sph2pipe\n");
+    return -1;
+}
+#endif /* !HAVE_POPEN */
+
+/**
+ * Detect NIST file and parse its header if detected.
+ *
+ * @return TRUE if it's a NIST file, FALSE if not, -1 if an error occurred.
+ */
+static int
+detect_nist(sphinx_wave2feat_t *wtf)
+{
+    FILE *fh;
+    int rv;
+
+    if ((rv = open_nist_file(wtf, wtf->infile, &fh, TRUE)) != TRUE)
+        return rv;
+    wtf->infh = fh;
+
+    return TRUE;
+}
 
 
 /**
@@ -202,15 +260,14 @@ detect_nist(sphinx_wave2feat_t *wtf, char const *infile)
  * @return TRUE, or -1 on error.
  */
 static int
-detect_raw(sphinx_wave2feat_t *wtf, char const *infile)
+detect_raw(sphinx_wave2feat_t *wtf)
 {
     FILE *fh;
 
-    if ((fh = fopen(infile, "rb")) == NULL) {
-        E_ERROR_SYSTEM("Failed to open %s", infile);
+    if ((fh = fopen(wtf->infile, "rb")) == NULL) {
+        E_ERROR_SYSTEM("Failed to open %s", wtf->infile);
         return -1;
     }
-    wtf->infile = ckd_salloc(infile);
     wtf->infh = fh;
     return TRUE;
 }
@@ -222,18 +279,19 @@ detect_raw(sphinx_wave2feat_t *wtf, char const *infile)
  * @return TRUE, or -1 on error.
  */
 static int
-detect_sphinx_mfc(sphinx_wave2feat_t *wtf, char const *infile)
+detect_sphinx_mfc(sphinx_wave2feat_t *wtf)
 {
     FILE *fh;
     int32 len;
     long flen;
 
-    if ((fh = fopen(infile, "rb")) == NULL) {
-        E_ERROR_SYSTEM("Failed to open %s", infile);
+    if ((fh = fopen(wtf->infile, "rb")) == NULL) {
+        E_ERROR_SYSTEM("Failed to open %s", wtf->infile);
         return -1;
     }
     if (fread(&len, 4, 1, fh) != 1) {
-        E_ERROR_SYSTEM("Failed to read header from %s\n", infile);
+        E_ERROR_SYSTEM("Failed to read header from %s\n", wtf->infile);
+        fclose(fh);
         return -1;
     }
     fseek(fh, 0, SEEK_END);
@@ -257,7 +315,6 @@ detect_sphinx_mfc(sphinx_wave2feat_t *wtf, char const *infile)
     }
     
     fseek(fh, 4, SEEK_SET);
-    wtf->infile = ckd_salloc(infile);
     wtf->infh = fh;
     if (cmd_ln_boolean_r(wtf->config, "-spec2cep")) {
         wtf->in_veclen = cmd_ln_int32_r(wtf->config, "-nfilt");
@@ -275,6 +332,107 @@ detect_sphinx_mfc(sphinx_wave2feat_t *wtf, char const *infile)
     return TRUE;
 }
 
+int
+mixnpick_channels(int16 *buf, int32 nsamp, int32 nchans, int32 whichchan)
+{
+    int i, j;
+
+    if (whichchan > 0) {
+        for (i = whichchan - 1; i < nsamp; i += nchans)
+            buf[i/nchans] = buf[i];
+    }
+    else {
+        for (i = 0; i < nsamp; i += nchans) {
+            float64 tmp = 0.0;
+            for (j = 0; j < nchans && i + j < nsamp; ++j) {
+                tmp += buf[i + j];
+            }
+            buf[i/nchans] = (int16)(tmp / nchans);
+        }
+    }
+    return i/nchans;
+}
+
+#ifdef HAVE_SNDFILE_H
+/**
+ * Detect a file supported by libsndfile and parse its header if detected.
+ *
+ * @return TRUE if it's a supported file, FALSE if not, -1 if an error occurred.
+ */
+static int
+detect_sndfile(sphinx_wave2feat_t *wtf)
+{
+    SNDFILE *sf;
+    SF_INFO sfinfo;
+
+    memset(&sfinfo, 0, sizeof(sfinfo));
+    /* We let other detectors catch I/O errors, since there is
+       no way to tell them from format errors when opening :( */
+    if ((sf = sf_open(wtf->infile, SFM_READ, &sfinfo)) == NULL) {
+        return FALSE;
+    }
+    /* Get relevant information. */
+    cmd_ln_set_int32_r(wtf->config, "-nchans", sfinfo.channels);
+    cmd_ln_set_float32_r(wtf->config, "-samprate", sfinfo.samplerate);
+    wtf->insfh = sf;
+    wtf->infh = NULL;
+
+    return TRUE;
+}
+
+/**
+ * Process PCM audio from a libsndfile file.  FIXME: looks a lot like
+ * decode_pcm!  Also needs stereo support (as does decode_pcm).
+ */
+static int
+decode_sndfile(sphinx_wave2feat_t *wtf)
+{
+    size_t nsamp;
+    int32 nfr, nchans, whichchan;
+    int nfloat, n;
+
+    nchans = cmd_ln_int32_r(wtf->config, "-nchans");
+    whichchan = cmd_ln_int32_r(wtf->config, "-whichchan");
+    fe_start_utt(wtf->fe);
+    nfloat = 0;
+    while ((nsamp = sf_read_short(wtf->insfh,
+                                  wtf->audio,
+                                  wtf->blocksize)) != 0) {
+        int16 const *inspeech;
+        size_t nvec;
+
+        /* Mix or pick channels. */
+        if (nchans > 1)
+            nsamp = mixnpick_channels(wtf->audio, nsamp, nchans, whichchan);
+
+        inspeech = wtf->audio;
+        nvec = wtf->featsize;
+        /* Consume all samples. */
+        while (nsamp) {
+            nfr = nvec;
+            fe_process_frames(wtf->fe, &inspeech, &nsamp, wtf->feat, &nfr);
+            if (nfr) {
+                if ((n = (*wtf->ot->output_frames)(wtf, wtf->feat, nfr)) < 0)
+                    return -1;
+                nfloat += n;
+            }
+        }
+        inspeech = wtf->audio;
+    }
+    /* Now process any leftover audio frames. */
+    fe_end_utt(wtf->fe, wtf->feat[0], &nfr);
+    if (nfr) {
+        if ((n = (*wtf->ot->output_frames)(wtf, wtf->feat, nfr)) < 0)
+            return -1;
+        nfloat += n;
+    }
+
+    sf_close(wtf->insfh);
+    wtf->insfh = NULL;
+    return nfloat;
+}
+#endif /* HAVE_SNDFILE_H */
+
 /**
  * Process PCM audio from a filehandle.  Assume that wtf->infh is
  * positioned just after the file header.
@@ -283,9 +441,11 @@ static int
 decode_pcm(sphinx_wave2feat_t *wtf)
 {
     size_t nsamp;
-    int32 nfr;
+    int32 nfr, nchans, whichchan;
     int nfloat, n;
 
+    nchans = cmd_ln_int32_r(wtf->config, "-nchans");
+    whichchan = cmd_ln_int32_r(wtf->config, "-whichchan");
     fe_start_utt(wtf->fe);
     nfloat = 0;
     while ((nsamp = fread(wtf->audio, 2, wtf->blocksize, wtf->infh)) != 0) {
@@ -297,6 +457,10 @@ decode_pcm(sphinx_wave2feat_t *wtf)
             for (n = 0; n < nsamp; ++n)
                 SWAP_INT16(wtf->audio + n);
         }
+
+        /* Mix or pick channels. */
+        if (nchans > 1)
+            nsamp = mixnpick_channels(wtf->audio, nsamp, nchans, whichchan);
             
         inspeech = wtf->audio;
         nvec = wtf->featsize;
@@ -320,7 +484,8 @@ decode_pcm(sphinx_wave2feat_t *wtf)
         nfloat += n;
     }
 
-    fclose(wtf->infh);
+    if (fclose(wtf->infh) == EOF)
+        E_ERROR_SYSTEM("Failed to close input file");
     wtf->infh = NULL;
     return nfloat;
 }
@@ -370,18 +535,20 @@ decode_sphinx_mfc(sphinx_wave2feat_t *wtf)
         nfloat += n;
     }
 
-    fclose(wtf->infh);
+    if (fclose(wtf->infh) == EOF)
+        E_ERROR_SYSTEM("Failed to close input file");
     wtf->infh = NULL;
     return nfloat;
 }
 
 static const audio_type_t types[] = {
-    { "-mswav", &detect_riff, &decode_pcm },
-    { "-nist", &detect_nist, &decode_pcm },
-#ifdef HAVE_SNDFILE
+#ifdef HAVE_SNDFILE_H
     { "-sndfile", &detect_sndfile, &decode_sndfile },
 #endif
-    { "-raw", &detect_raw, &decode_pcm }
+    { "-mswav", &detect_riff, &decode_pcm },
+    { "-nist", &detect_nist, &decode_pcm },
+    { "-raw", &detect_raw, &decode_pcm },
+    { "-sph2pipe", &detect_sph2pipe, &decode_pcm }
 };
 static const int ntypes = sizeof(types)/sizeof(types[0]);
 static const audio_type_t mfcc_type = {
@@ -592,14 +759,22 @@ sphinx_wave2feat_free(sphinx_wave2feat_t *wtf)
     if (--wtf->refcount > 0)
         return wtf->refcount;
 
-    ckd_free(wtf->audio);
-    ckd_free_2d(wtf->feat);
-    ckd_free(wtf->infile);
-    ckd_free(wtf->outfile);
-    if (wtf->infh)
-        fclose(wtf->infh);
-    if (wtf->outfh)
-        fclose(wtf->outfh);
+    if (wtf->audio)
+	ckd_free(wtf->audio);
+    if (wtf->feat)
+	ckd_free_2d(wtf->feat);
+    if (wtf->infile)
+        ckd_free(wtf->infile);
+    if (wtf->outfile)
+	ckd_free(wtf->outfile);
+    if (wtf->infh) {
+        if (fclose(wtf->infh) == EOF)
+            E_ERROR_SYSTEM("Failed to close input file");
+    }
+    if (wtf->outfh) {
+        if (fclose(wtf->outfh) == EOF)
+            E_ERROR_SYSTEM("Failed to close output file");
+    }
     cmd_ln_free_r(wtf->config);
     fe_free(wtf->fe);
     ckd_free(wtf);
@@ -615,7 +790,7 @@ sphinx_wave2feat_retain(sphinx_wave2feat_t *wtf)
 }
 
 static audio_type_t const *
-detect_audio_type(sphinx_wave2feat_t *wtf, char const *infile)
+detect_audio_type(sphinx_wave2feat_t *wtf)
 {
     audio_type_t const *atype;
     int i;
@@ -623,9 +798,9 @@ detect_audio_type(sphinx_wave2feat_t *wtf, char const *infile)
     /* Special case audio type for Sphinx MFCC inputs. */
     if (cmd_ln_boolean_r(wtf->config, "-spec2cep")
         || cmd_ln_boolean_r(wtf->config, "-cep2spec")) {
-        int rv = mfcc_type.detect(wtf, infile);
+        int rv = mfcc_type.detect(wtf);
         if (rv == -1)
-            return NULL;
+            goto error_out;
         return &mfcc_type;
     }
 
@@ -634,10 +809,11 @@ detect_audio_type(sphinx_wave2feat_t *wtf, char const *infile)
         int rv;
         atype = &types[i];
         if (cmd_ln_boolean_r(wtf->config, atype->name)) {
-            rv = (*atype->detect)(wtf, infile);
+            rv = (*atype->detect)(wtf);
             if (rv == -1)
-                return NULL;
-            break;
+                goto error_out;
+            else if (rv == TRUE)
+                break;
         }
     }
     if (i == ntypes) {
@@ -645,31 +821,38 @@ detect_audio_type(sphinx_wave2feat_t *wtf, char const *infile)
         for (i = 0; i < ntypes; ++i) {
             int rv;
             atype = &types[i];
-            rv = (*atype->detect)(wtf, infile);
+            rv = (*atype->detect)(wtf);
             if (rv == -1)
-                return NULL;
+                goto error_out;
             else if (rv == TRUE)
-            break;
+                break;
         }
         if (i == ntypes)
-            atype = NULL;
+            goto error_out;
     }
     return atype;
+ error_out:
+    if (wtf->infh)
+        fclose(wtf->infh);
+    wtf->infh = NULL;
+    return NULL;
 }
 
 int
 sphinx_wave2feat_convert_file(sphinx_wave2feat_t *wtf,
                               char const *infile, char const *outfile)
 {
-    int minfft, nfft, nfloat, veclen;
+    int nchans, minfft, nfft, nfloat, veclen;
     audio_type_t const *atype;
     int fshift, fsize;
 
     if (cmd_ln_boolean_r(wtf->config, "-verbose"))
         E_INFO("Converting %s to %s\n", infile, outfile);
 
+    wtf->infile = ckd_salloc(infile);
+
     /* Detect input file type. */
-    if ((atype = detect_audio_type(wtf, infile)) == NULL)
+    if ((atype = detect_audio_type(wtf)) == NULL)
         return -1;
 
     /* Determine whether to byteswap input. */
@@ -695,19 +878,24 @@ sphinx_wave2feat_convert_file(sphinx_wave2feat_t *wtf,
 
     /* Set up the input and output buffers. */
     fe_get_input_size(wtf->fe, &fshift, &fsize);
-    /* Want to get at least a whole frame plus shift in here. */
-    wtf->blocksize = cmd_ln_int32_r(wtf->config, "-blocksize");
-    if (wtf->blocksize < fsize + fshift) {
+    /* Want to get at least a whole frame plus shift in here.  Also we
+       will either pick or mix multiple channels so we need to read
+       them all at once. */
+    nchans = cmd_ln_int32_r(wtf->config, "-nchans");
+    wtf->blocksize = cmd_ln_int32_r(wtf->config, "-blocksize") * nchans;
+    if (wtf->blocksize < (fsize + fshift) * nchans) {
         E_INFO("Block size of %d too small, increasing to %d\n",
-               wtf->blocksize, fsize + fshift);
-        wtf->blocksize = fsize + fshift;
+               wtf->blocksize,
+               (fsize + fshift) * nchans);
+        wtf->blocksize = (fsize + fshift) * nchans;
     }
     wtf->audio = ckd_calloc(wtf->blocksize, sizeof(*wtf->audio));
-    wtf->featsize = (wtf->blocksize - fsize) / fshift;
+    wtf->featsize = (wtf->blocksize / nchans - fsize) / fshift;
 
     /* Use the maximum of the input and output frame sizes to allocate this. */
     veclen = wtf->veclen;
     if (wtf->in_veclen > veclen) veclen = wtf->in_veclen;
+    
     wtf->feat = ckd_calloc_2d(wtf->featsize, veclen, sizeof(**wtf->feat));
 
     /* Let's go! */
@@ -723,8 +911,10 @@ sphinx_wave2feat_convert_file(sphinx_wave2feat_t *wtf,
     }
     wtf->outfile = ckd_salloc(outfile);
 
-    if ((nfloat = (*atype->decode)(wtf)) < 0)
-        return -1;
+    if ((nfloat = (*atype->decode)(wtf)) < 0) {
+    	E_ERROR("Failed to convert");
+    	goto error_out;
+    }
 
     if (wtf->ot->output_header) {
         if (fseek(wtf->outfh, 0, SEEK_SET) < 0) {
@@ -736,15 +926,50 @@ sphinx_wave2feat_convert_file(sphinx_wave2feat_t *wtf,
             goto error_out;
         }
     }
-    fclose(wtf->outfh);
+    
+
+    if (wtf->audio)
+	ckd_free(wtf->audio);
+    if (wtf->feat)
+	ckd_free_2d(wtf->feat);
+    if (wtf->infile)
+        ckd_free(wtf->infile);
+    if (wtf->outfile)
+	ckd_free(wtf->outfile);
+
+    wtf->audio = NULL;
+    wtf->infile = NULL;
+    wtf->feat = NULL;
+    wtf->outfile = NULL;
+
+    if (wtf->outfh)
+	if (fclose(wtf->outfh) == EOF)
+    	    E_ERROR_SYSTEM("Failed to close output file");
     wtf->outfh = NULL;
 
     return 0;
+
 error_out:
-    if (wtf->outfh) {
-        fclose(wtf->outfh);
-        wtf->outfh = NULL;
-    }
+
+    if (wtf->audio)
+	ckd_free(wtf->audio);
+    if (wtf->feat)
+	ckd_free_2d(wtf->feat);
+    if (wtf->infile)
+        ckd_free(wtf->infile);
+    if (wtf->outfile)
+	ckd_free(wtf->outfile);
+
+    wtf->audio = NULL;
+    wtf->infile = NULL;
+    wtf->feat = NULL;
+    wtf->outfile = NULL;
+
+    if (wtf->outfh)
+	if (fclose(wtf->outfh) == EOF)
+    	    E_ERROR_SYSTEM("Failed to close output file");
+    wtf->outfh = NULL;
+
     return -1;
 }
 
@@ -784,9 +1009,11 @@ build_filenames(cmd_ln_t *config, char const *basename,
 static int
 run_control_file(sphinx_wave2feat_t *wtf, char const *ctlfile)
 {
+    hash_table_t *files;
+    hash_iter_t *itor;
     lineiter_t *li;
     FILE *ctlfh;
-    int nskip, runlen, npart;
+    int nskip, runlen, npart, rv = 0;
 
     if ((ctlfh = fopen(ctlfile, "r")) == NULL) {
         E_ERROR_SYSTEM("Failed to open control file %s", ctlfile);
@@ -796,9 +1023,9 @@ run_control_file(sphinx_wave2feat_t *wtf, char const *ctlfile)
     runlen = cmd_ln_int32_r(wtf->config, "-runlen");
     if ((npart = cmd_ln_int32_r(wtf->config, "-npart"))) {
         /* Count lines in the file. */
-        int nlines, partlen, part;
+        int partlen, part, nlines = 0;
         part = cmd_ln_int32_r(wtf->config, "-part");
-        for (nlines = 0, li = lineiter_start(ctlfh); li; li = lineiter_next(li))
+        for (li = lineiter_start(ctlfh); li; li = lineiter_next(li))
             ++nlines;
         fseek(ctlfh, 0, SEEK_SET);
         partlen = nlines / npart;
@@ -808,32 +1035,53 @@ run_control_file(sphinx_wave2feat_t *wtf, char const *ctlfile)
         else
             runlen = partlen;
     }
-    if (runlen != -1)
+    if (runlen != -1){
         E_INFO("Processing %d utterances at position %d\n", runlen, nskip);
-    else
+        files = hash_table_new(runlen, HASH_CASE_YES);
+    }
+    else {
         E_INFO("Processing all remaining utterances at position %d\n", nskip);
+        files = hash_table_new(1000, HASH_CASE_YES);
+    }
     for (li = lineiter_start(ctlfh); li; li = lineiter_next(li)) {
-        char *infile, *outfile;
-        int rv;
+        char *c, *infile, *outfile;
 
         if (nskip-- > 0)
             continue;
-        if (runlen == 0)
+        if (runlen == 0) {
+            lineiter_free(li);
             break;
+        }
         --runlen;
 
         string_trim(li->buf, STRING_BOTH);
+        /* Extract the file ID from the control line. */
+        if ((c = strchr(li->buf, ' ')) != NULL)
+            *c = '\0';
+        if (strlen(li->buf) == 0) {
+    	    E_WARN("Empty line %d in control file, skipping\n", li->lineno);
+    	    continue;
+        }
         build_filenames(wtf->config, li->buf, &infile, &outfile);
+        if (hash_table_lookup(files, infile, NULL) == 0)
+            continue;
         rv = sphinx_wave2feat_convert_file(wtf, infile, outfile);
-        ckd_free(infile);
-        ckd_free(outfile);
+        hash_table_enter(files, infile, outfile);
         if (rv != 0) {
             lineiter_free(li);
-            fclose(ctlfh);
-            return rv;
+            break;
         }
     }
-    return 0;
+    for (itor = hash_table_iter(files); itor;
+         itor = hash_table_iter_next(itor)) {
+        ckd_free((void *)hash_entry_key(itor->ent));
+        ckd_free(hash_entry_val(itor->ent));
+    }
+    hash_table_free(files);
+
+    if (fclose(ctlfh) == EOF)
+        E_ERROR_SYSTEM("Failed to close control file");
+    return rv;
 }
 
 int
@@ -870,5 +1118,6 @@ main(int argc, char *argv[])
                                            cmd_ln_str_r(config, "-o"));
 
     sphinx_wave2feat_free(wtf);
+    cmd_ln_free_r(config);
     return rv;
 }
