@@ -35,13 +35,63 @@
  *
  */
 
-#include <hash_table.h>
-#include <fsg_model.h>
-#include <jsgf.h>
-#include <err.h>
-
 #include <string.h>
-#include <strfuncs.h>
+
+#include <sphinxbase/hash_table.h>
+#include <sphinxbase/fsg_model.h>
+#include <sphinxbase/jsgf.h>
+#include <sphinxbase/err.h>
+#include <sphinxbase/strfuncs.h>
+
+static const arg_t defn[] = {
+  { "-help",
+    ARG_BOOLEAN,
+    "no",
+    "Shows the usage of the tool"},
+
+  { "-jsgf",
+    REQARG_STRING,
+    NULL,
+    "Input grammar in jsgf format (required)"},
+
+  { "-rule",
+    ARG_STRING,
+    NULL,
+    "Root rule name (optional)"},
+
+  { "-fsg",
+    ARG_STRING,
+    NULL,
+    "Output grammar in fsg format"},
+
+  { "-fsm",
+    ARG_STRING,
+    NULL,
+    "Output grammar in FSM format"},
+
+  { "-symtab",
+    ARG_STRING,
+    NULL,
+    "Output symtab for grammar in FSM format"},
+
+  { "-compile",
+    ARG_BOOLEAN,
+    "no",
+    "Compute grammar closure to speedup loading"},
+
+  { NULL, 0, NULL, NULL }
+};
+
+
+static void
+usagemsg(char *pgm)
+{
+    E_INFO("Usage: %s -jsgf <input.jsgf> -rule <rule name>\\\n", pgm);
+    E_INFOCONT("\t[-fsm yes/no] [-compile yes/no]\n");
+    E_INFOCONT("\t-fsg <output.fsg>\n");
+
+    exit(0);
+}
 
 static fsg_model_t *
 get_fsg(jsgf_t *grammar, const char *name)
@@ -73,27 +123,30 @@ main(int argc, char *argv[])
 {
     jsgf_t *jsgf;
     fsg_model_t *fsg;
-    int fsm = 0;
-    char *outfile = NULL;
-    char *symfile = NULL;
-
-    if (argc > 1 && 0 == strcmp(argv[1], "-fsm")) {
-        fsm = 1;
-        ++argv;
+    cmd_ln_t *config;
+        
+    if ((config = cmd_ln_parse_r(NULL, defn, argc, argv, TRUE)) == NULL)
+	return 1;
+		
+    if (cmd_ln_boolean_r(config, "-help")) {
+        usagemsg(argv[0]);
     }
 
-    jsgf = jsgf_parse_file(argc > 1 ? argv[1] : NULL, NULL);
+    jsgf = jsgf_parse_file(cmd_ln_str_r(config, "-jsgf"), NULL);
     if (jsgf == NULL) {
         return 1;
     }
-    fsg = get_fsg(jsgf, argc > 2 ? argv[2] : NULL);
 
-    if (argc > 3)
-        outfile = argv[3];
-    if (argc > 4)
-        symfile = argv[4];
+    fsg = get_fsg(jsgf, cmd_ln_str_r(config, "-rule") ? cmd_ln_str_r(config, "-rule") : NULL);
+    
+    if (cmd_ln_boolean_r(config, "-compile")) {
+	fsg_model_null_trans_closure(fsg, NULL);
+    }
 
-    if (fsm) {
+    
+    if (cmd_ln_str_r(config, "-fsm")) {
+	const char* outfile = cmd_ln_str_r(config, "-fsm");
+	const char* symfile = cmd_ln_str_r(config, "-symtab");
         if (outfile)
             fsg_model_writefile_fsm(fsg, outfile);
         else
@@ -102,6 +155,7 @@ main(int argc, char *argv[])
             fsg_model_writefile_symtab(fsg, symfile);
     }
     else {
+        const char *outfile = cmd_ln_str_r(config, "-fsg");
         if (outfile)
             fsg_model_writefile(fsg, outfile);
         else
@@ -114,7 +168,6 @@ main(int argc, char *argv[])
 }
 
 
-/** Silvio Moioli: Windows CE/Mobile entry point added. */
 #if defined(_WIN32_WCE)
 #pragma comment(linker,"/entry:mainWCRTStartup")
 #include <windows.h>
