@@ -11,7 +11,7 @@ JSGF_PATH=$tests/regression
 export JSGF_PATH
 for r in $rules; do
     run_program sphinx_jsgf2fsg/sphinx_jsgf2fsg \
-	$tests/regression/test.gram $r > $r.out 2>>$tmpout
+	-jsgf $tests/regression/test.gram -rule $r -fsg $r.out 2>>$tmpout
     compare_table $r $r.out $tests/regression/$r.fsg
 done
 
